(* ::Package:: *)

(*:Package Version: 3.01 *)

(*:Version History: 1.01 (2013.05.05 - 2013.05.24)
                    1.02 (2013.05.25 - 2014.04.15)
                    2.01 (2014.04.16 - 2014.04.22)Lame
                    2.02 (2014.04.23 - 2014.07.31)LameWave
                    2.03 (2014.08.01 - 2014.11.11)LameLC,LS
                    2.04 (2014.11.12 - 2015.01.04)HC,HC2,Chop
                    3.01 (2015.01.05 -     .  .  )Eigenvalue *)

(*:Copyright: Copyright 2013.05-2015.01, Souichiro Ikebe *)

(*:Author: Souichiro Ikebe *)

(*:Name: SpecialFunctions`NDSolveHill` *)

(*:Title: NDSolve, Hill, Lame, LameWave *)

(*:Keywords: Hill, HillPrime, HillEllipticTheta, HillEllipticThetaPrime, 
             HillSinCos, HillSinCosPrime, HillCosSin, HillCosSinPrime, 
             HillCosCos, HillCosCosPrime, MeissnerHill, MeissnerHillPrime, 
             LameEigenvalueHC, LameEigenvalueHS, LameEigenvalueHC2, LameEigenvalueHS2, 
             LameEigenvalueA, LameEigenvalueB, LameEigenvalueA2, LameEigenvalueB2, 
             LameCoefficientA, LameCoefficientB, LameCoefficientC, LameCoefficientD, 
             LameEC, LameES, LameECPrime, LameESPrime, 
             LameFC, LameFS, LameFCPrime, LameFSPrime, 
             LameLC, LameLS, LameLCPrime, LameLSPrime, 
             LamePolynomialUE, LamePolynomialSE, LamePolynomialCE, 
             LamePolynomialDE, LamePolynomialSCE, LamePolynomialSDE, 
             LamePolynomialCDE, LamePolynomialSCDE, 
             LamePolynomialUEPrime, LamePolynomialSEPrime, LamePolynomialCEPrime, 
             LamePolynomialDEPrime, LamePolynomialSCEPrime, LamePolynomialSDEPrime, 
             LamePolynomialCDEPrime, LamePolynomialSCDEPrime, 
             GeneralizedLame, GeneralizedLamePrime, 
             GeneralizedLameWave, GeneralizedLameWavePrime *)

(*:Sources:
  A.Erdelyi, W.Magnus, F.Oberhettinger, F.G.Tricomi,
  "Higher Transcendental Functions Vol.3", 
  McGrawHill, New York (1955),pp .62-91

  J.K.M.Jansen, 
  "Simple periodic and non-periodic Lame functions", 
  Mathematical centre tracts #72.

  E.L.Ince, 
  "The periodic Lame functions", 
  Proceedings of the Royal Society of Edinburgh, #60 (1940),pp .47-63.

  E.L.Ince, 
  "Further investigations into the periodic Lame functions", 
  Proceedings of the Royal Society of Edinburgh, #60 (1940),pp .83-99. 

  H.Volkmer, G.Wolf,(edited: F.W.J.Olver, D.W.Lozier, R.F.Boisvert, C.W.Clark),
  "NIST Handbook of mathematical functions",
  National institute of standards and technology & 
  Cambridge university press, (2010),pp .651-681 & 683-695.
*)

(*:Summary:
\:3000\:3000This package will add a method to solved in the complex domain 
\:3000\:3000the solution of Ellipsoidal differential equations using the NDSolve, 
\:3000\:3000a feature built into the Mathematica.
   This package allows the calculation of Hill-functions, Lame-functions, 
   Lame-(Ellipsoidal) wave functions, etc...
*)

(**********************************************************************************)





BeginPackage["SpecialFunctions`NDSolveHill`"]





Hill::usage =
"Hill[a,b,z,{w1,w2},p[#]&] gives solutions of the Hill's differential 
 equation : y''[z]+(a-b*p[z])*y[z]=0; with y[0]=w1; y'[0]=w2."

HillPrime::usage =
"HillPrime[a,b,z,{w1,w2},p[#]&] gives derivative of the Hill's function : 
 Hill[a,b,z,{w1,w2},p[#]&] at z."

HillEllipticTheta::usage =
"HillEllipticTheta[a,b,{j,z,q},{w1,w2}] gives solutions of the Hill's 
 differential equation : y''[z]+(a-b*EllipticTheta[j,2z,q])*y[z]=0; 
 with y[0]=w1; y'[0]=w2; j=2,3,4."

HillEllipticThetaPrime::usage =
"HillEllipticThetaPrime[a,b,{j,z,q},{w1,w2}] gives derivative of the 
 Hill's function : HillEllipticTheta[a,b,{j,z,q},{w1,w2}] at z."

HillSinCos::usage =
"HillSinCos[a,b,z,{w1,w2}] gives solutions of the Hill's differential 
 equation : y''[z]+(a-b*Sin[Cos[z]])*y[z]=0; with y[0]=w1; y'[0]=w2."

HillSinCosPrime::usage =
"HillSinCosPrime[a,b,z,{w1,w2}] gives derivative of the Hill's function : 
 HillSinCos[a,b,z,{w1,w2}] at z."

HillCosSin::usage =
"HillCosSin[a,b,z,{w1,w2}] gives solutions of the Hill's differential 
 equation : y''[z]+(a-b*Cos[Sin[z]])*y[z]=0; with y[0]=w1; y'[0]=w2."

HillCosSinPrime::usage =
"HillCosSinPrime[a,b,z,{w1,w2}] gives derivative of the Hill's function : 
 HillCosSin[a,b,z,{w1,w2}] at z."

HillCosCos::usage =
"HillCosCos[a,b,z,{w1,w2}] gives solutions of the Hill's differential 
 equation : y''[z]+(a-b*Cos[Cos[z]])*y[z]=0; with y[0]=w1; y'[0]=w2."

HillCosCosPrime::usage =
"HillCosCosPrime[a,b,z,{w1,w2}] gives derivative of the Hill's function : 
 HillCosCos[a,b,z,{w1,w2}] at z."

MeissnerHill::usage =
"MeissnerHill[a,b,z,{w1,w2}] gives solutions of the Meissner-Hill's 
 differential equation : 
 y''[z]+(a-b*Sign[Cos[z]])*y[z]=0; with y[0]=w1; y'[0]=w2."

MeissnerHillPrime::usage =
"MeissnerHillPrime[a,b,z,{w1,w2}] gives derivative of the Meissner-Hill's 
 function : MeissnerHill[a,b,z,{w1,w2}] at z."

LameEigenvalueHC::usage =
"LameEigenvalueHC[mu,v,k] gives eigenvalues for Fourier coefficients of the 
 even (Ec, Fc, etc.) Lame functions."

LameEigenvalueHS::usage =
"LameEigenvalueHS[mu,v,k] gives eigenvalues for Fourier coefficients of the 
 odd (Es, Fs, etc.) Lame functions."

LameEigenvalueHC2::usage =
"LameEigenvalueHC2[mu,v,k] gives other branch cut of the eigenvalues : 
 LameEigenvalueHC[mu,v,k]."

LameEigenvalueHS2::usage =
"LameEigenvalueHS2[mu,v,k] gives other branch cut of the eigenvalues : 
 LameEigenvalueHS[mu,v,k]."

LameEigenvalueA::usage =
"LameEigenvalueA[mu,v,k] gives eigenvalues h in Lame equation : 
 y''[z]+{h-v(v+1)k*sn(z,k)^2}*y[z]=0 for the even (Ec,Fc, etc.) Lame functions."

LameEigenvalueB::usage =
"LameEigenvalueB[mu,v,k] gives eigenvalues h in Lame equation : 
 y''[z]+{h-v(v+1)k*sn(z,k)^2}*y[z]=0 for the odd (Es,Fs, etc.) Lame functions."

LameEigenvalueA2::usage =
"LameEigenvalueA2[mu,v,k] gives other branch cut of the eigenvalues : 
 LameEigenvalueA[mu,v,k]."

LameEigenvalueB2::usage =
"LameEigenvalueB2[mu,v,k] gives other branch cut of the eigenvalues : 
 LameEigenvalueB[mu,v,k]."

LameCoefficientA::usage =
"LameCoefficientA[n,v,k,m] gives coefficients of the Fourier series on even 
 (Ec, Fc, etc.) Lame functions."

LameCoefficientB::usage =
"LameCoefficientB[n,v,k,m] gives coefficients of the Fourier series on odd 
 (Es, Fs, etc.) Lame functions."

LameCoefficientC::usage =
"LameCoefficientC[n,v,k,m] gives coefficients of the Fourier series on even 
 (dE, sdE) Lame polynomials."

LameCoefficientD::usage =
"LameCoefficientD[n,v,k,m] gives coefficients of the Fourier series on odd 
 (cdE, scdE) Lame polynomials."

LameEC::usage =
"LameEC[v,n,{z,k}] gives even Lame functions of the first kind."

LameES::usage =
"LameES[v,n,{z,k}] gives odd Lame functions of the first kind."

LameECPrime::usage =
"LameECPrime[v,n,{z,k}] gives derivative of even Lame functions of the 
 first kind at z."

LameESPrime::usage =
"LameESPrime[v,n,{z,k}] gives derivative of odd Lame functions of the 
 first kind at z."

LameFC::usage =
"LameFC[v,n,{z,k}] gives even Lame functions of the second kind."

LameFS::usage =
"LameFS[v,n,{z,k}] gives odd Lame functions of the second kind."

LameFCPrime::usage =
"LameFCPrime[v,n,{z,k}] gives derivative of even Lame functions of the 
 second kind at z."

LameFSPrime::usage =
"LameFSPrime[v,n,{z,k}] gives derivative of odd Lame functions of the 
 second kind at z."

LameLC::usage =
"LameLC[v,n,{z,k}] gives even Lame functions of the third kind,
 LameLC[v,n,{z,k}]=LameEC[v,n,{z,k}]+I*LameFC[v,n,{z,k}]."

LameLS::usage =
"LameLS[v,n,{z,k}] gives odd Lame functions of the third kind,
 LameLS[v,n,{z,k}]=LameES[v,n,{z,k}]+I*LameFS[v,n,{z,k}]."

LameLCPrime::usage =
"LameLCPrime[v,n,{z,k}] gives derivative of even Lame functions of the 
 third kind at z."

LameLSPrime::usage =
"LameLSPrime[v,n,{z,k}] gives derivative of odd Lame functions of the 
 third kind at z."

LamePolynomialUE::usage =
"LamePolynomialUE[v,m,{z,k}] gives even (uE) Lame polynomials."

LamePolynomialSE::usage =
"LamePolynomialSE[v,m,{z,k}] gives even (sE) Lame polynomials."

LamePolynomialCE::usage =
"LamePolynomialCE[v,m,{z,k}] gives odd (cE) Lame polynomials."

LamePolynomialDE::usage =
"LamePolynomialDE[v,m,{z,k}] gives even (dE) Lame polynomials."

LamePolynomialSCE::usage =
"LamePolynomialSCE[v,m,{z,k}] gives odd (scE) Lame polynomials."

LamePolynomialSDE::usage =
"LamePolynomialSDE[v,m,{z,k}] gives even (sdE) Lame polynomials."

LamePolynomialCDE::usage =
"LamePolynomialCDE[v,m,{z,k}] gives odd (cdE) Lame polynomials."

LamePolynomialSCDE::usage =
"LamePolynomialSCDE[v,m,{z,k}] gives odd (scdE) Lame polynomials."

LamePolynomialUEPrime::usage =
"LamePolynomialUEPrime[v,m,{z,k}] gives derivative of even (uE) Lame 
 polynomials at z."

LamePolynomialSEPrime::usage =
"LamePolynomialSEPrime[v,m,{z,k}] gives derivative of even (sE) Lame 
 polynomials at z."

LamePolynomialCEPrime::usage =
"LamePolynomialCEPrime[v,m,{z,k}] gives derivative of odd (cE) Lame 
 polynomials at z."

LamePolynomialDEPrime::usage =
"LamePolynomialDEPrime[v,m,{z,k}] gives derivative of even (dE) Lame 
 polynomials at z."

LamePolynomialSCEPrime::usage =
"LamePolynomialSCEPrime[v,m,{z,k}] gives derivative of odd (scE) Lame 
 polynomials at z."

LamePolynomialSDEPrime::usage =
"LamePolynomialSDEPrime[v,m,{z,k}] gives derivative of even (sdE) Lame 
 polynomials at z."

LamePolynomialCDEPrime::usage =
"LamePolynomialCDEPrime[v,m,{z,k}] gives derivative of odd (cdE) Lame 
 polynomials at z."

LamePolynomialSCDEPrime::usage =
"LamePolynomialSCDEPrime[v,m,{z,k}] gives derivative of odd (scdE) Lame 
 polynomials at z."

GeneralizedLame::usage =
"GeneralizedLame[h,v,{z,k},{w0,w1}] gives solutions of the Lame differential 
 equation : y''[z]+(h-v(v+1)k*JacobiSN[z,k]^2])*y[z]=0; 
 with y[0]=w0; y'[0]=w1."

GeneralizedLamePrime::usage =
"GeneralizedLamePrime[h,v,{z,k},{w0,w1}] gives derivative of the Lame 
 function : GeneralizedLame[h,v,{z,k},{w0,w1}] at z."

GeneralizedLameWave::usage =
"GeneralizedLameWave[h,q,v,{z,k},{w0,w1}] gives solutions of the Lame-wave differential 
 equation : y''[z]+(h-v(v+1)k*JacobiSN[z,k]^2]+q*k^2*JacobiSN[z,k]^4)*y[z]=0; 
 with y[0]=w0; y'[0]=w1."

GeneralizedLameWavePrime::usage =
"GeneralizedLameWavePrime[h,q,v,{z,k},{w0,w1}] gives derivative of the Lame-wave 
 function : GeneralizedLameWave[h,q,v,{z,k},{w0,w1}] at z."





Begin["SpecialFunctions`NDSolveHill`Private`"]





(**********************************************************************************)
(*                                  Common codes                                  *)
(**********************************************************************************)

intNQ[n_]:=
  If[N[Re[n]-Round[Re[n]]]===0. && N[Im[n]]===0.,True,False];



(**********************************************************************************)
(*                           Hill[a,b,z,{w1,w2},p[#]&],                           *)
(*                         HillPrime[a,b,z,{w1,w2},p[#]&]                         *)
(**********************************************************************************)

realHill[a_,b_,z_,{w1_,w2_},phi_]:=
  Module[{sol,f,x},
    sol=NDSolve[{f''[x]+(a-b*phi[x])*f[x]==0,
      f[0]==w1,f'[0]==w2},f,{x,z}];
    {f[z],f'[z]}/.sol[[1]]];

Hill[a_,b_,z_,{w1_,w2_},phi_]:=
  Module[{init,sol,f,x},
    init=realHill[a,b,Re[z],{w1,w2},phi];
    sol=NDSolve[{f''[x]-(a-b*phi[Re[z]+x*I])*f[x]==0,
      f[0]==init[[1]],f'[0]==I*init[[2]]},f,{x,Im[z]}];
    f[Im[z]]/.sol[[1]]]/;
   (Precision[z]<Infinity)

HillPrime[a_,b_,z_,{w1_,w2_},phi_]:=
  Module[{init,sol,f,x},
    init=realHill[a,b,Re[z],{w1,w2},phi];
    sol=NDSolve[{f''[x]-(a-b*phi[Re[z]+x*I])*f[x]==0,
      f[0]==init[[1]],f'[0]==I*init[[2]]},f,{x,Im[z]}];
    -I*f'[Im[z]]/.sol[[1]]]/;
   (Precision[z]<Infinity)



(**********************************************************************************)
(*                     HillEllipticTheta[a,b,{j,z,q},{w1,w2}],                    *)
(*                  HillEllipticThetaPrime[a,b,{j,z,q},{w1,w2}]                   *)
(**********************************************************************************)

realHillETheta[a_,b_,{j_,z_,q_},{w1_,w2_}]:=
  Module[{sol,f,x},
    sol=NDSolve[{f''[x]+(a-b*EllipticTheta[j,2 x,q])*f[x]==0,
      f[0]==w1,f'[0]==w2},f,{x,z}];
    {f[z],f'[z]}/.sol[[1]]];

HillEllipticTheta[a_,b_,{j_,z_,q_},{w1_,w2_}]:=
  Module[{init,sol,f,x},
    init=realHillETheta[a,b,{j,Re[z],q},{w1,w2}];
    sol=NDSolve[{f''[x]-(a-b*EllipticTheta[j,2 (Re[z]+x*I),q])*f[x]==0,
      f[0]==init[[1]],f'[0]==I*init[[2]]},f,{x,Im[z]}];
    f[Im[z]]/.sol[[1]]]/;
   (Precision[z]<Infinity)

HillEllipticThetaPrime[a_,b_,{j_,z_,q_},{w1_,w2_}]:=
  Module[{init,sol,f,x},
    init=realHillETheta[a,b,{j,Re[z],q},{w1,w2}];
    sol=NDSolve[{f''[x]-(a-b*EllipticTheta[j,2 (Re[z]+x*I),q])*f[x]==0,
      f[0]==init[[1]],f'[0]==I*init[[2]]},f,{x,Im[z]}];
    -I*f'[Im[z]]/.sol[[1]]]/;
   (Precision[z]<Infinity)



(**********************************************************************************)
(*                           HillSinCos[a,b,z,{w1,w2}],                           *)
(*                         HillSinCosPrime[a,b,z,{w1,w2}]                         *)
(**********************************************************************************)

realHillSinCos[a_,b_,z_,{w1_,w2_}]:=
  Module[{sol,f,x},
    sol=NDSolve[{f''[x]+(a-b*Sin[Cos[x]])*f[x]==0,
      f[0]==w1,f'[0]==w2},f,{x,z}];
    {f[z],f'[z]}/.sol[[1]]];

HillSinCos[a_,b_,z_,{w1_,w2_}]:=
  Module[{init,sol,f,x},
    init=realHillSinCos[a,b,Re[z],{w1,w2}];
    sol=NDSolve[{f''[x]-(a-b*Sin[Cos[Re[z]+x*I]])*f[x]==0,
      f[0]==init[[1]],f'[0]==I*init[[2]]},f,{x,Im[z]}];
    f[Im[z]]/.sol[[1]]]/;
   (Precision[z]<Infinity)

HillSinCosPrime[a_,b_,z_,{w1_,w2_}]:=
  Module[{init,sol,f,x},
    init=realHillSinCos[a,b,Re[z],{w1,w2}];
    sol=NDSolve[{f''[x]-(a-b*Sin[Cos[Re[z]+x*I]])*f[x]==0,
      f[0]==init[[1]],f'[0]==I*init[[2]]},f,{x,Im[z]}];
   -I*f'[Im[z]]/.sol[[1]]]/;
   (Precision[z]<Infinity)



(**********************************************************************************)
(*                           HillCosSin[a,b,z,{w1,w2}],                           *)
(*                         HillCosSinPrime[a,b,z,{w1,w2}]                         *)
(**********************************************************************************)

realHillCosSin[a_,b_,z_,{w1_,w2_}]:=
  Module[{sol,f,x},
    sol=NDSolve[{f''[x]+(a-b*Cos[Sin[x]])*f[x]==0,
      f[0]==w1,f'[0]==w2},f,{x,z}];
    {f[z],f'[z]}/.sol[[1]]];

HillCosSin[a_,b_,z_,{w1_,w2_}]:=
  Module[{init,sol,f,x},
    init=realHillCosSin[a,b,Re[z],{w1,w2}];
    sol=NDSolve[{f''[x]-(a-b*Cos[Sin[Re[z]+x*I]])*f[x]==0,
      f[0]==init[[1]],f'[0]==I*init[[2]]},f,{x,Im[z]}];
    f[Im[z]]/.sol[[1]]]/;
   (Precision[z]<Infinity)

HillCosSinPrime[a_,b_,z_,{w1_,w2_}]:=
  Module[{init,sol,f,x},
    init=realHillCosSin[a,b,Re[z],{w1,w2}];
    sol=NDSolve[{f''[x]-(a-b*Cos[Sin[Re[z]+x*I]])*f[x]==0,
      f[0]==init[[1]],f'[0]==I*init[[2]]},f,{x,Im[z]}];
    -I*f'[Im[z]]/.sol[[1]]]/;
   (Precision[z]<Infinity)



(**********************************************************************************)
(*                           HillCosCos[a,b,z,{w1,w2}],                           *)
(*                         HillCosCosPrime[a,b,z,{w1,w2}]                         *)
(**********************************************************************************)

realHillCosCos[a_,b_,z_,{w1_,w2_}]:=
  Module[{sol,f,x},
    sol=NDSolve[{f''[x]+(a-b*Cos[Cos[x]])*f[x]==0,
      f[0]==w1,f'[0]==w2},f,{x,z}];
    {f[z],f'[z]}/.sol[[1]]];

HillCosCos[a_,b_,z_,{w1_,w2_}]:=
  Module[{init,sol,f,x},
    init=realHillCosCos[a,b,Re[z],{w1,w2}];
    sol=NDSolve[{f''[x]-(a-b*Cos[Cos[Re[z]+x*I]])*f[x]==0,
      f[0]==init[[1]],f'[0]==I*init[[2]]},f,{x,Im[z]}];
    f[Im[z]]/.sol[[1]]]/;
   (Precision[z]<Infinity)

HillCosCosPrime[a_,b_,z_,{w1_,w2_}]:=
  Module[{init,sol,f,x},
    init=realHillCosCos[a,b,Re[z],{w1,w2}];
    sol=NDSolve[{f''[x]-(a-b*Cos[Cos[Re[z]+x*I]])*f[x]==0,
      f[0]==init[[1]],f'[0]==I*init[[2]]},f,{x,Im[z]}];
    -I*f'[Im[z]]/.sol[[1]]]/;
   (Precision[z]<Infinity)



(**********************************************************************************)
(*                          MeissnerHill[a,b,z,{w1,w2}],                          *)
(*                        MeissnerHillPrime[a,b,z,{w1,w2}]                        *)
(**********************************************************************************)

realMeissnerHill[a_,b_,z_,{w1_,w2_}]:=
  Module[{sol,f,x},
    sol=NDSolve[{f''[x]+(a-b*Sign[Cos[x]])*f[x]==0,
      f[0]==w1,f'[0]==w2},f,{x,z},MaxStepSize->0.05,StartingStepSize->0.01];
    {f[z],f'[z]}/.sol[[1]]];

MeissnerHill[a_,b_,z_,{w1_,w2_}]:=
  Module[{init,sol,f,x},
    init=realMeissnerHill[a,b,Re[z],{w1,w2}];
    sol=NDSolve[{f''[x]-(a-b*Sign[Cos[Re[z]+x*I]])*f[x]==0,
      f[0]==init[[1]],f'[0]==I*init[[2]]},f,{x,Im[z]}];
    f[Im[z]]/.sol[[1]]]/;
   (Precision[z]<Infinity)

MeissnerHillPrime[a_,b_,z_,{w1_,w2_}]:=
  Module[{init,sol,f,x},
    init=realMeissnerHill[a,b,Re[z],{w1,w2}];
    sol=NDSolve[{f''[x]-(a-b*Sign[Cos[Re[z]+x*I]])*f[x]==0,
      f[0]==init[[1]],f'[0]==I*init[[2]]},f,{x,Im[z]}];
   -I*f'[Im[z]]/.sol[[1]]]/;
   (Precision[z]<Infinity)



(**********************************************************************************)
(*                                Lame eigenvalues                                *)
(**********************************************************************************)

(*---------------- Coefficients of recurrence relation ----------------*)

(* Integer order use *)
alpHCe[p_,v_,k_]:=
  Which[N[p]===0.,(v-1) (v+2) k,p>0,(v-2 p-1) (v+2 p+2) k/2];
betHCe[p_,v_,k_]:=4*p^2*(2-k);
gamHCe[p_,v_,k_]:=(v-2 p+2) (v+2 p-1) k/2;

alpHCo[p_,v_,k_]:=(v-2 p-2) (v+2 p+3) k/2;
betHCo[p_,v_,k_]:=
  Which[N[p]===0.,2-k+v (v+1) k/2,p>0,(2 p+1)^2*(2-k)];
gamHCo[p_,v_,k_]:=(v-2 p+1) (v+2 p) k/2;

alpHSe[p_,v_,k_]:=(v-2 p-3) (v+2 p+4) k/2;
betHSe[p_,v_,k_]:=(2 p+2)^2*(2-k);
gamHSe[p_,v_,k_]:=(v-2 p) (v+2 p+1) k/2;

alpHSo[p_,v_,k_]:=(v-2 p-2) (v+2 p+3) k/2;
betHSo[p_,v_,k_]:=
  Which[N[p]===0.,2-k-v (v+1) k/2,p>0,(2 p+1)^2*(2-k)];
gamHSo[p_,v_,k_]:=(v-2 p+1) (v+2 p) k/2;

(* Not integer order use *)
alpHC[p_,v_,k_]:=(v-2 p-1) (v+2 p+2) k/2;
betHC[p_,v_,k_]:=4*p^2*(2-k);
gamHC[p_,v_,k_]:=(v-2 p+2) (v+2 p-1) k/2;

(*-------------- Lame eigenvalues "Hc","Hs"(n = Integer) --------------*)

(* Max of matrix size *)
matSizeLame[n_,v_]:=
  Module[{av=Round[Sqrt[Abs[v]]],rn=Round[Abs[n]]},
  10+av+rn];

(* Eigenmatrix *)
matLameHCe[mMax_,v_,k_]:=
  Table[Switch[p-q,-1,alpHCe[p,v,k],0,betHCe[p,v,k],1,gamHCe[p,v,k],_,0],
  {p,0,mMax},{q,0,mMax}];

matLameHCo[mMax_,v_,k_]:=
  Table[Switch[p-q,-1,alpHCo[p,v,k],0,betHCo[p,v,k],1,gamHCo[p,v,k],_,0],
  {p,0,mMax},{q,0,mMax}];

matLameHSe[mMax_,v_,k_]:=
  Table[Switch[p-q,-1,alpHSe[p,v,k],0,betHSe[p,v,k],1,gamHSe[p,v,k],_,0],
  {p,0,mMax},{q,0,mMax}];

matLameHSo[mMax_,v_,k_]:=
  Table[Switch[p-q,-1,alpHSo[p,v,k],0,betHSo[p,v,k],1,gamHSo[p,v,k],_,0],
  {p,0,mMax},{q,0,mMax}];

(* Lame eigenvalue Hc[n,v][k], Hs[n,v][k] *)
LameEigenvalueHC[n_,v_,k_]:=
  Module[{mMax,mat},
    mMax=matSizeLame[n,v];
    mat=If[EvenQ[Round[n]],matLameHCe[mMax,v,k],matLameHCo[mMax,v,k]];
    Sort[Eigenvalues[N[mat]]][[Ceiling[(n+1/2)/2]]]]/;
  (intNQ[n] && Re[n]>=0 && Precision[v]<Infinity)

LameEigenvalueHS[n_,v_,k_]:=
  Module[{mMax,mat},
    mMax=matSizeLame[n,v];
    mat=If[EvenQ[Round[n]],matLameHSe[mMax,v,k],matLameHSo[mMax,v,k]];
    Sort[Eigenvalues[N[mat]]][[Ceiling[(n-1/2)/2]]]]/;
  (intNQ[n] && Re[n]>=0 && Precision[v]<Infinity)

(*----------- Lame eigenvalues "Hc","Hs"(mu = Not integer) ------------*)

(* Eigenmatrix *)
matLameHC[mMax_,mu_,v_,k_]:=
  Table[Switch[p-q,-1,alpHC[p+mu/2,v,k],0,betHC[p+mu/2,v,k],1,gamHC[p+mu/2,v,k],_,0],
  {p,-mMax,mMax},{q,-mMax,mMax}];

(* Lame eigenvalue Hc[mu,v][k], Hs[mu,v][k] *)
LameEigenvalueHC[mu_,v_,k_]:=
  Module[{mMax,mat},
    mMax=matSizeLame[mu/4,v];
    mat=matLameHC[mMax,mu,v,k];
    Chop[Sort[Eigenvalues[N[mat]]][[Floor[Re[mu+1]]]],10^-8]]/;
  (Not[intNQ[mu]] && Re[mu]>=0 && Precision[v]<Infinity)

LameEigenvalueHS[mu_,v_,k_]:=
  LameEigenvalueHC[mu,v,k]/;
  (Not[intNQ[mu]] && Re[mu]>=0 && Precision[v]<Infinity)

(* Parity of mu *)

LameEigenvalueHC[mu_,v_,k_]:=
  LameEigenvalueHC[-mu,v,k]/;(Re[mu]<0)

LameEigenvalueHS[mu_,v_,k_]:=
  LameEigenvalueHS[-mu,v,k]/;(Re[mu]<0)

(*------------- Lame eigenvalues "Hc2","Hs2"(n = Integer) -------------*)

LameEigenvalueHC2[n_,v_,k_]:=
  LameEigenvalueHS[n,v,k]/;
  (intNQ[n] && Re[n]>=0 && Precision[v]<Infinity)

LameEigenvalueHS2[n_,v_,k_]:=
  LameEigenvalueHC[n,v,k]/;
  (intNQ[n] && Re[n]>=0 && Precision[v]<Infinity)

(*---------- Lame eigenvalues "Hc2","Hs2"(mu = Not integer) -----------*)

LameEigenvalueHC2[mu_,v_,k_]:=
  Module[{mMax,mat},
    mMax=matSizeLame[mu/4,v];
    mat=matLameHC[mMax,mu,v,k];
    Chop[Sort[Eigenvalues[N[mat]]][[If[Re[mu]>0,Floor[Re[mu]],1]]],10^-8]]/;
  (Not[intNQ[mu]] && Re[mu]>=0 && Precision[v]<Infinity)

LameEigenvalueHS2[mu_,v_,k_]:=
  LameEigenvalueHC2[mu,v,k]/;
  (Not[intNQ[mu]] && Re[mu]>=0 && Precision[v]<Infinity)

(* Parity of mu *)

LameEigenvalueHC2[mu_,v_,k_]:=
  LameEigenvalueHC2[-mu,v,k]/;(Re[mu]<0)

LameEigenvalueHS2[mu_,v_,k_]:=
  LameEigenvalueHS2[-mu,v,k]/;(Re[mu]<0)

(*--------------------- Lame eigenvalues "A","B" ----------------------*)

LameEigenvalueA[n_,v_,k_]:=
  (LameEigenvalueHC[n,v,k]+v (v+1) k)/2/;
  (Precision[v]<Infinity)

LameEigenvalueB[n_,v_,k_]:=
  (LameEigenvalueHS[n,v,k]+v (v+1) k)/2/;
  (Precision[v]<Infinity)

(*-------------------- Lame eigenvalues "A2","B2" ---------------------*)

LameEigenvalueA2[n_,v_,k_]:=
  (LameEigenvalueHC2[n,v,k]+v (v+1) k)/2/;
  (Precision[v]<Infinity)

LameEigenvalueB2[n_,v_,k_]:=
  (LameEigenvalueHS2[n,v,k]+v (v+1) k)/2/;
  (Precision[v]<Infinity)

(*----------------- Lame eigenvalues : Precision type -----------------*)

setPrecLame[x_]:=SetPrecision[x,150];

(* Lame eigenvalue Hc[n,v][k], Hs[n,v][k] *)
precLameEigenvalueHC[n_Integer,v_,k_]:=
  Module[{pN,mMax,mat},
    pN=If[n>=0,n,-n];
    mMax=20+matSizeLame[pN,v];
    mat=If[EvenQ[pN],matLameHCe[mMax,v,k],matLameHCo[mMax,v,k]];
    Sort[Eigenvalues[setPrecLame[mat]]][[Ceiling[(pN+1/2)/2]]]];

precLameEigenvalueHS[n_Integer,v_,k_]:=
  Module[{pN,mMax,mat},
    pN=If[n>=0,n,-n];
    mMax=20+matSizeLame[pN,v];
    mat=If[EvenQ[pN],matLameHSe[mMax,v,k],matLameHSo[mMax,v,k]];
    Sort[Eigenvalues[setPrecLame[mat]]][[Ceiling[(pN-1/2)/2]]]];

(* Lame eigenvalue a[n,v][k], b[n,v][k] *)
precLameEigenvalueA[n_Integer,v_,k_]:=
  (precLameEigenvalueHC[n,v,k]+v (v+1) k)/2;

precLameEigenvalueB[n_Integer,v_,k_]:=
  (precLameEigenvalueHS[n,v,k]+v (v+1) k)/2;



(**********************************************************************************)
(*                                 Lame functions                                 *)
(**********************************************************************************)

(*----------------- Coefficients of Fourier series A ------------------*)

(* Eigenvalues HC *)
leHC[n_,v_,k_]:=
  leHC[n,v,k]=precLameEigenvalueHC[n,v,k];

(* Coefficients Ae[n,v,k][m] *)
coeffLVe[n_,v_,k_,0]:=
  (leHC[n,v,k]-betHCe[0,v,k])/alpHCe[0,v,k];

coeffLVe[n_,v_,k_,m_/;(EvenQ[m] && m>0)]:=
  Module[{clve=coeffLVe[n,v,k,m-2]},
    If[N[clve]===0.,0,
     (leHC[n,v,k]-betHCe[m/2,v,k]-gamHCe[m/2,v,k]/clve)/alpHCe[m/2,v,k]]];

coeffLVe[n_,v_,k_,m_/;Not[EvenQ[m] && m>=0]]:=0;

coeffLameVe[n_,v_,k_,m_]:=
  coeffLameVe[n,v,k,m]=
    If[EvenQ[n] && OddQ[Round[v]] && Abs[v-Round[v]]<10^-8,
     coeffLVe[n,v+10^-9,k,m],coeffLVe[n,v,k,m]];

coeffLameAe[n_,v_,k_,0]:=
  Module[{s=0,so=1,h=2,zf},
    zf=coeffLameVe[n,v,k,0];
    While[so!=s,so=s;
      s+=zf^2;
      zf*=coeffLameVe[n,v,k,h];h+=2];
    1/Sqrt[s+1/2]];

coeffLameAe[n_,v_,k_,m_/;(EvenQ[m] && m>0)]:=
  Product[coeffLameVe[n,v,k,h],{h,0,m-2,2}]*coeffLameAe[n,v,k,0];

coeffLameAe[n_,v_,k_,m_/;Not[EvenQ[m] && m>=0]]:=0;

(* Coefficients Ao[n,v,k][m] *)
coeffLVo[n_,v_,k_,1]:=
  (leHC[n,v,k]-betHCo[0,v,k])/alpHCo[0,v,k];

coeffLVo[n_,v_,k_,m_/;(OddQ[m] && m>1)]:=
  Module[{clvo=coeffLVo[n,v,k,m-2]},
    If[N[clvo]===0.,0,
     (leHC[n,v,k]-betHCo[(m-1)/2,v,k]-gamHCo[(m-1)/2,v,k]/clvo)/alpHCo[(m-1)/2,v,k]]];

coeffLVo[n_,v_,k_,m_/;Not[OddQ[m] && m>=1]]:=0;

coeffLameVo[n_,v_,k_,m_]:=
  coeffLameVo[n,v,k,m]=
    If[OddQ[n] && EvenQ[Round[v]] && Abs[v-Round[v]]<10^-8,
     coeffLVo[n,v+10^-9,k,m],coeffLVo[n,v,k,m]];

coeffLameAo[n_,v_,k_,1]:=
  Module[{s=0,so=1,h=3,zf},
    zf=coeffLameVo[n,v,k,1];
    While[so!=s,so=s;
      s+=zf^2;
      zf*=coeffLameVo[n,v,k,h];h+=2];
    1/Sqrt[s+1]];

coeffLameAo[n_,v_,k_,m_/;(OddQ[m] && m>1)]:=
  Product[coeffLameVo[n,v,k,h],{h,1,m-2,2}]*coeffLameAo[n,v,k,1];

coeffLameAo[n_,v_,k_,m_/;Not[OddQ[m] && m>=1]]:=0;

(* Coefficients A[n,v,k][m] *)
LameCoefficientA[n_Integer?NonNegative,v_,k_,m_Integer]:=
  LameCoefficientA[n,v,k,m]=
    Which[EvenQ[n] && EvenQ[m],coeffLameAe[n,v,k,m],
         OddQ[n] && OddQ[m],coeffLameAo[n,v,k,m]];

(*----------------- Coefficients of Fourier series B ------------------*)

(* Eigenvalues HS *)
leHS[n_,v_,k_]:=
  leHS[n,v,k]=precLameEigenvalueHS[n,v,k];

(* Coefficients Be[n,v,k][m] *)
coeffLWe[n_,v_,k_,2]:=
  (leHS[n,v,k]-betHSe[0,v,k])/alpHSe[0,v,k];

coeffLWe[n_,v_,k_,m_/;(EvenQ[m] && m>2)]:=
  Module[{clwe=coeffLWe[n,v,k,m-2]},
    If[N[clwe]===0.,0,
     (leHS[n,v,k]-betHSe[(m-2)/2,v,k]-gamHSe[(m-2)/2,v,k]/clwe)/alpHSe[(m-2)/2,v,k]]];

coeffLWe[n_,v_,k_,m_/;Not[EvenQ[m] && m>=2]]:=0;

coeffLameWe[n_,v_,k_,m_]:=
  coeffLameWe[n,v,k,m]=
    If[EvenQ[n] && OddQ[Round[v]] && Re[v]>1 && Abs[v-Round[v]]<10^-8,
     coeffLWe[n,v+10^-9,k,m],coeffLWe[n,v,k,m]];

coeffLameBe[n_,v_,k_,2]:=
  Module[{s=0,so=1,h=4,zf},
    zf=coeffLameWe[n,v,k,2];
    While[so!=s,so=s;
      s+=zf^2;
      zf*=coeffLameWe[n,v,k,h];h+=2];
    1/Sqrt[s+1]];

coeffLameBe[n_,v_,k_,m_/;(EvenQ[m] && m>2)]:=
  Product[coeffLameWe[n,v,k,h],{h,2,m-2,2}]*coeffLameBe[n,v,k,2];

coeffLameBe[n_,v_,k_,m_/;Not[EvenQ[m] && m>=2]]:=0;

(* Coefficients Bo[n,v,k][m] *)
coeffLWo[n_,v_,k_,1]:=
  (leHS[n,v,k]-betHSo[0,v,k])/alpHSo[0,v,k];

coeffLWo[n_,v_,k_,m_/;(OddQ[m] && m>1)]:=
  Module[{clwo=coeffLWo[n,v,k,m-2]},
  If[N[clwo]===0.,0,
    (leHS[n,v,k]-betHSo[(m-1)/2,v,k]-gamHSo[(m-1)/2,v,k]/clwo)/alpHSo[(m-1)/2,v,k]]];

coeffLWo[n_,v_,k_,m_/;Not[OddQ[m] && m>=1]]:=0;

coeffLameWo[n_,v_,k_,m_]:=
  coeffLameWo[n,v,k,m]=
    If[OddQ[n] && EvenQ[Round[v]] && Re[v]>1 && Abs[v-Round[v]]<10^-8,
     coeffLWo[n,v+10^-9,k,m],coeffLWo[n,v,k,m]];

coeffLameBo[n_,v_,k_,1]:=
  Module[{s=0,so=1,h=3,zf},
    zf=coeffLameWo[n,v,k,1];
    While[so!=s,so=s;
      s+=zf^2;
      zf*=coeffLameWo[n,v,k,h];h+=2];
    1/Sqrt[s+1]];

coeffLameBo[n_,v_,k_,m_/;(OddQ[m] && m>1)]:=
  Product[coeffLameWo[n,v,k,h],{h,1,m-2,2}]*coeffLameBo[n,v,k,1];

coeffLameBo[n_,v_,k_,m_/;Not[OddQ[m] && m>=1]]:=0;

(* Coefficients B[n,v,k][m] *)
LameCoefficientB[n_Integer?Positive,v_,k_,m_Integer]:=
  LameCoefficientB[n,v,k,m]=
    Which[EvenQ[n] && EvenQ[m],coeffLameBe[n,v,k,m],
         OddQ[n] && OddQ[m],coeffLameBo[n,v,k,m]];

(*------------------ Lame polynomials matching test -------------------*)

lameCPolyQ[v_,n_]:=
  If[IntegerQ[n] && N[v-Round[v]]===0. && 0<=n<=v,
  True,False];

lameSPolyQ[v_,n_]:=
  If[IntegerQ[n] && N[v-Round[v]]===0. && 1<=n<=v,
  True,False];

(*---------------- Initial values of Lame function Ec -----------------*)

initLameEC[v_,n_Integer?NonNegative,{z_,k_}]:=
  Module[{s=0,so=1,h=1,vp,zp,kp,phi},
    vp=setPrecLame[v];
    zp=setPrecLame[z];
    kp=setPrecLame[k];
    phi=setPrecLame[Pi/2-JacobiAmplitude[zp,kp]];
    While[(Abs[so-s]>10^-15 || h<30),so=s;
      s+=LameCoefficientA[n,vp,kp,2 h]*Cos[(2 h)*phi];
      h++];
    N[s+LameCoefficientA[n,vp,kp,0]/2]]*sgnFactorC[v,n,k]/;
  (Not[lameCPolyQ[v,n]] && EvenQ[n] && Re[v]>=-1/2 && Precision[z]<Infinity)

initLameEC[v_,n_Integer?NonNegative,{z_,k_}]:=
  Module[{s=0,so=1,h=0,vp,zp,kp,phi},
    vp=setPrecLame[v];
    zp=setPrecLame[z];
    kp=setPrecLame[k];
    phi=setPrecLame[Pi/2-JacobiAmplitude[zp,kp]];
    While[(Abs[so-s]>10^-15 || h<30),so=s;
      s+=LameCoefficientA[n,vp,kp,2 h+1]*Cos[(2 h+1)*phi];
      h++];
    N[s]]*sgnFactorC[v,n,k]/;
  (Not[lameCPolyQ[v,n]] && OddQ[n] && Re[v]>=-1/2 && Precision[z]<Infinity)

initLameEC[v_,n_Integer?NonNegative,{z_,k_}]:=
initLameEC[-v-1,n,{z,k}]/;
(Re[v]<-1/2 && Precision[z]<Infinity)

(* sgnFactorC[v,n,k] *)
sgnFactorC[v_,n_Integer?NonNegative,k_]:=
  Module[{vp,kp,valECK},
    vp=setPrecLame[v];
    kp=setPrecLame[k];
    valECK=Which[EvenQ[n],
      Sum[LameCoefficientA[n,vp,kp,2 h],{h,1,30}]+LameCoefficientA[n,vp,kp,0]/2,
        OddQ[n],
      Sum[LameCoefficientA[n,vp,kp,2 h+1],{h,0,30}]];
    If[Re[valECK]>0,1,-1]]/;
  (Re[v]>=-1/2)

(* Polynomials case *)
initLameEC[v_,n_Integer?NonNegative,{z_,k_}]:=
  If[EvenQ[n],
    If[EvenQ[Round[v]],
      LamePolynomialUE[v,n/2,{z,k}],LamePolynomialDE[v,n/2,{z,k}]],
    If[EvenQ[Round[v]],
      LamePolynomialSDE[v,(n-1)/2,{z,k}],LamePolynomialSE[v,(n-1)/2,{z,k}]]]/;
  (lameCPolyQ[v,n] && Re[v]>=-1/2 && Precision[z]<Infinity)

(*---------------- Initial values of Lame function Es -----------------*)

initLameES[v_,n_Integer?Positive,{z_,k_}]:=
  Module[{s=0,so=1,h=1,vp,zp,kp,phi},
    vp=setPrecLame[v];
    zp=setPrecLame[z];
    kp=setPrecLame[k];
    phi=setPrecLame[Pi/2-JacobiAmplitude[zp,kp]];
    While[(Abs[so-s]>10^-15 || h<30),so=s;
      s+=LameCoefficientB[n,vp,kp,2 h]*Sin[(2 h)*phi];
      h++];
    N[s]]*sgnFactorS[v,n,k]/;
  (Not[lameSPolyQ[v,n]] && EvenQ[n] && Re[v]>=-1/2 && Precision[z]<Infinity)

initLameES[v_,n_Integer?Positive,{z_,k_}]:=
  Module[{s=0,so=1,h=0,vp,zp,kp,phi},
    vp=setPrecLame[v];
    zp=setPrecLame[z];
    kp=setPrecLame[k];
    phi=setPrecLame[Pi/2-JacobiAmplitude[zp,kp]];
    While[(Abs[so-s]>10^-15 || h<30),so=s;
      s+=LameCoefficientB[n,vp,kp,2 h+1]*Sin[(2 h+1)*phi];
      h++];
    N[s]]*sgnFactorS[v,n,k]/;
  (Not[lameSPolyQ[v,n]] && OddQ[n] && Re[v]>=-1/2 && Precision[z]<Infinity)

initLameES[v_,n_Integer?Positive,{z_,k_}]:=
  initLameES[-v-1,n,{z,k}]/;
  (Re[v]<-1/2 && Precision[z]<Infinity)

(* sgnFactorS[v,n,k] *)
sgnFactorS[v_,n_Integer?Positive,k_]:=
  Module[{vp,kp,valESK},
    vp=setPrecLame[v];
    kp=setPrecLame[k];
    valESK=Which[EvenQ[n],
      Sqrt[1-k]*Sum[2 h*LameCoefficientB[n,vp,kp,2 h],{h,1,30}],
        OddQ[n],
      Sqrt[1-k]*Sum[(2 h+1)*LameCoefficientB[n,vp,kp,2 h+1],{h,0,30}]];
  If[Re[valESK]>0,1,-1]]/;(Re[v]>=-1/2)

(* Polynomials case *)
initLameES[v_,n_Integer?Positive,{z_,k_}]:=
  If[EvenQ[n],
    If[EvenQ[Round[v]],
      LamePolynomialSCE[v,n/2-1,{z,k}],LamePolynomialSCDE[v,n/2-1,{z,k}]],
    If[EvenQ[Round[v]],
      LamePolynomialCDE[v,(n-1)/2,{z,k}],LamePolynomialCE[v,(n-1)/2,{z,k}]]]/;
  (lameSPolyQ[v,n] && Re[v]>=-1/2 && Precision[z]<Infinity)

(*----------- Initial values of Derivative Lame function Ec -----------*)

initLameECPrime[v_,n_Integer?NonNegative,{z_,k_}]:=
  Module[{s=0,so=1,h=1,vp,zp,kp,phi},
    vp=setPrecLame[v];
    zp=setPrecLame[z];
    kp=setPrecLame[k];
    phi=setPrecLame[Pi/2-JacobiAmplitude[zp,kp]];
    While[(Abs[so-s]>10^-15 || h<30),so=s;
      s+=2 h*LameCoefficientA[n,vp,kp,2 h]*Sin[(2 h)*phi];
      h++];
    N[s]]*sgnFactorC[v,n,k]*JacobiDN[z,k]/;
  (Not[lameCPolyQ[v,n]] && EvenQ[n] && Re[v]>=-1/2 && Precision[z]<Infinity)

initLameECPrime[v_,n_Integer?NonNegative,{z_,k_}]:=
  Module[{s=0,so=1,h=0,vp,zp,kp,phi},
    vp=setPrecLame[v];
    zp=setPrecLame[z];
    kp=setPrecLame[k];
    phi=setPrecLame[Pi/2-JacobiAmplitude[zp,kp]];
    While[(Abs[so-s]>10^-15 || h<30),so=s;
      s+=(2 h+1)*LameCoefficientA[n,vp,kp,2 h+1]*Sin[(2 h+1)*phi];
      h++];
    N[s]]*sgnFactorC[v,n,k]*JacobiDN[z,k]/;
  (Not[lameCPolyQ[v,n]] && OddQ[n] && Re[v]>=-1/2 && Precision[z]<Infinity)

initLameECPrime[v_,n_Integer?NonNegative,{z_,k_}]:=
  initLameECPrime[-v-1,n,{z,k}]/;
  (Re[v]<-1/2 && Precision[z]<Infinity)

(* Polynomials case *)
initLameECPrime[v_,n_Integer?NonNegative,{z_,k_}]:=
  If[EvenQ[n],
    If[EvenQ[Round[v]],
      LamePolynomialUEPrime[v,n/2,{z,k}],LamePolynomialDEPrime[v,n/2,{z,k}]],
    If[EvenQ[Round[v]],
      LamePolynomialSDEPrime[v,(n-1)/2,{z,k}],LamePolynomialSEPrime[v,(n-1)/2,{z,k}]]]/;
  (lameCPolyQ[v,n] && Re[v]>=-1/2 && Precision[z]<Infinity)

(*----------- Initial values of Derivative Lame function Es -----------*)

initLameESPrime[v_,n_Integer?Positive,{z_,k_}]:=
  Module[{s=0,so=1,h=1,vp,zp,kp,phi},
    vp=setPrecLame[v];
    zp=setPrecLame[z];
    kp=setPrecLame[k];
    phi=setPrecLame[Pi/2-JacobiAmplitude[zp,kp]];
    While[(Abs[so-s]>10^-15 || h<30),so=s;
      s+=(-2 h)*LameCoefficientB[n,vp,kp,2 h]*Cos[(2 h)*phi];
      h++];
    N[s]]*sgnFactorS[v,n,k]*JacobiDN[z,k]/;
  (Not[lameSPolyQ[v,n]] && EvenQ[n] && Re[v]>=-1/2 && Precision[z]<Infinity)

initLameESPrime[v_,n_Integer?Positive,{z_,k_}]:=
  Module[{s=0,so=1,h=0,vp,zp,kp,phi},
    vp=setPrecLame[v];
    zp=setPrecLame[z];
    kp=setPrecLame[k];
    phi=setPrecLame[Pi/2-JacobiAmplitude[zp,kp]];
    While[(Abs[so-s]>10^-15 || h<30),so=s;
      s+=(-2 h-1)*LameCoefficientB[n,vp,kp,2 h+1]*Cos[(2 h+1)*phi];
      h++];
    N[s]]*sgnFactorS[v,n,k]*JacobiDN[z,k]/;
 (Not[lameSPolyQ[v,n]] && OddQ[n] && Re[v]>=-1/2 && Precision[z]<Infinity)

initLameESPrime[v_,n_Integer?Positive,{z_,k_}]:=
 initLameESPrime[-v-1,n,{z,k}]/;
 (Re[v]<-1/2 && Precision[z]<Infinity)

(* Polynomials case *)
initLameESPrime[v_,n_Integer?Positive,{z_,k_}]:=
  If[EvenQ[n],
    If[EvenQ[Round[v]],
      LamePolynomialSCEPrime[v,n/2-1,{z,k}],LamePolynomialSCDEPrime[v,n/2-1,{z,k}]],
    If[EvenQ[Round[v]],
      LamePolynomialCDEPrime[v,(n-1)/2,{z,k}],LamePolynomialCEPrime[v,(n-1)/2,{z,k}]]]/;
  (lameSPolyQ[v,n] && Re[v]>=-1/2 && Precision[z]<Infinity)

(*---------------- Jacobi's Elliptic function sn[z,k] -----------------*)

(* The sn[z,k] is calculate faster than JacobiSN[z,k]. *)
sn[z_,k_]:=
  Module[{q=EllipticNomeQ[k],x=Pi*z/(2*EllipticK[k]),cons},
    cons=EllipticTheta[3,0.,q]/EllipticTheta[2,0.,q];
    cons*EllipticTheta[1,x,q]/EllipticTheta[4,x,q]];

(*---------- "NDSolve" : Methods for values of Lame function ----------*)

Lame[h_,v_,{z_,k_},{w0_,w1_}]:=
  Module[{sol,f,x},
    sol=NDSolve[{f''[x]+(h-v*(v+1)*k*sn[x,k]^2)*f[x]==0,
      f[0]==w0,f'[0]==w1},f,{x,z},MaxStepSize->0.01];
    f[Re[z]]/.sol[[1]]];

LamePrime[h_,v_,{z_,k_},{w0_,w1_}]:=
  Module[{sol,f,x},
    sol=NDSolve[{f''[x]+(h-v*(v+1)*k*sn[x,k]^2)*f[x]==0,
      f[0]==w0,f'[0]==w1},f,{x,z},MaxStepSize->0.01];
    f'[Re[z]]/.sol[[1]]];

(*--------- "NDSolve" of Lame function : Initial point at z=K ---------*)

LameK[h_,v_,{z_,k_},{w0_,w1_}]:=
  Module[{theta,sol,f,x,shift=N[EllipticK[k]]},
    theta=If[z==shift,0,Arg[z-shift]];
    sol=NDSolve[{Exp[-2 I*theta]*f''[x]+(h-v*(v+1)*k*sn[Exp[I*theta]*x+shift,k]^2)*f[x]==0,
      f[0]==w0,f'[0]==Exp[I*theta]*w1},f,{x,Abs[z-shift]}];
    f[Abs[z-shift]]/.sol[[1]]]/;
  (Precision[z]<Infinity)

LamePrimeK[h_,v_,{z_,k_},{w0_,w1_}]:=
  Module[{theta,sol,f,x,shift=N[EllipticK[k]]},
    theta=If[z==shift,0,Arg[z-shift]];
    sol=NDSolve[{Exp[-2 I*theta]*f''[x]+(h-v*(v+1)*k*sn[Exp[I*theta]*x+shift,k]^2)*f[x]==0,
      f[0]==w0,f'[0]==Exp[I*theta]*w1},f,{x,Abs[z-shift]}];
    Exp[-I*theta]*f'[Abs[z-shift]]/.sol[[1]]]/;
  (Precision[z]<Infinity)

(*---------------------- Speed up : Eigenvalues -----------------------*)

leA[n_,v_,k_]:=
  leA[n,v,k]=precLameEigenvalueA[n,v,k];

leB[n_,v_,k_]:=
  leB[n,v,k]=precLameEigenvalueB[n,v,k];

(*--------------------- Speed up : Initial values ---------------------*)
initECK[v_,n_,k_]:=
  initECK[v,n,k]=initLameEC[v,n,{N[EllipticK[k]],k}];

initESK[v_,n_,k_]:=
  initESK[v,n,k]=initLameES[v,n,{N[EllipticK[k]],k}];

initECPrimeK[v_,n_,k_]:=
  initECPrimeK[v,n,k]=initLameECPrime[v,n,{N[EllipticK[k]],k}];

initESPrimeK[v_,n_,k_]:=
  initESPrimeK[v,n,k]=initLameESPrime[v,n,{N[EllipticK[k]],k}];

initEC0[v_,n_,k_]:=
  initEC0[v,n,k]=initLameEC[v,n,{0.,k}];

initES0[v_,n_,k_]:=
  initES0[v,n,k]=initLameES[v,n,{0.,k}];

initECPrime0[v_,n_,k_]:=
  initECPrime0[v,n,k]=initLameECPrime[v,n,{0.,k}];

initESPrime0[v_,n_,k_]:=
  initESPrime0[v,n,k]=initLameESPrime[v,n,{0.,k}];

initFCK[v_,n_,k_]:=
  initFCK[v,n,k]=initLameFC[v,n,{N[EllipticK[k]],k}];

initFSK[v_,n_,k_]:=
  initFSK[v,n,k]=initLameFS[v,n,{N[EllipticK[k]],k}];

initFCPrimeK[v_,n_,k_]:=
  initFCPrimeK[v,n,k]=initLameFCPrime[v,n,{N[EllipticK[k]],k}];

initFSPrimeK[v_,n_,k_]:=
  initFSPrimeK[v,n,k]=initLameFSPrime[v,n,{N[EllipticK[k]],k}];

(*-------------------- Lame function Ec[v,n][z,k] ---------------------*)
LameEC[v_,n_Integer?NonNegative,{z_,k_}]:=
  Module[{w0,w1},
    w0=initECK[v,n,k];
    w1=initECPrimeK[v,n,k];
    If[Abs[Im[z]]>0,
      LameK[leA[n,v,k],v,{z,k},{w0,w1}],
      initLameEC[v,n,{z,k}]]]/;
  (Not[lameCPolyQ[v,n]] && 0<=Re[z]<=N[2*EllipticK[k]] && Precision[z]<Infinity)

LameEC[v_,n_Integer?NonNegative,{z_,k_}]:=
  Module[{eK,pmN,zK,pK},
    eK=N[EllipticK[k]];
    pmN=If[EvenQ[n],1,-1];
    zK=2*eK-Mod[-Re[z],2*eK]+I*Im[z];
    pK=Ceiling[Re[z]/(2*eK)-1];
    pmN^pK*LameEC[v,n,{zK,k}]]/;
  (Not[lameCPolyQ[v,n]] && Re[z]>N[2*EllipticK[k]] && Precision[z]<Infinity)

LameEC[v_,n_Integer?NonNegative,{z_,k_}]:=
  Module[{eK,pmN,zK,pK},
    eK=N[EllipticK[k]];
    pmN=If[EvenQ[n],1,-1];
    zK=Mod[Re[z],2*eK]+I*Im[z];
    pK=-Floor[Re[z]/(2*eK)];
    pmN^pK*LameEC[v,n,{zK,k}]]/;
  (Not[lameCPolyQ[v,n]] && Re[z]<0 && Precision[z]<Infinity)

(* Polynomials case *)
LameEC[v_,n_Integer?NonNegative,{z_,k_}]:=
  If[EvenQ[n],
    If[EvenQ[Round[v]],
      LamePolynomialUE[v,n/2,{z,k}],
      LamePolynomialDE[v,n/2,{z,k}]],
    If[EvenQ[Round[v]],
      LamePolynomialSDE[v,(n-1)/2,{z,k}],
    LamePolynomialSE[v,(n-1)/2,{z,k}]]]/;
  (lameCPolyQ[v,n] && Precision[z]<Infinity)

(*-------------------- Lame function Es[v,n][z,k] ---------------------*)
LameES[v_,n_Integer?Positive,{z_,k_}]:=
  Module[{w0,w1},
    w0=initESK[v,n,k];
    w1=initESPrimeK[v,n,k];
    If[Abs[Im[z]]>0,
      LameK[leB[n,v,k],v,{z,k},{w0,w1}],
      initLameES[v,n,{z,k}]]]/;
  (Not[lameSPolyQ[v,n]] && 0<=Re[z]<=N[2*EllipticK[k]] && Precision[z]<Infinity)

LameES[v_,n_Integer?Positive,{z_,k_}]:=
  Module[{eK,pmN,zK,pK},
    eK=N[EllipticK[k]];
    pmN=If[EvenQ[n],1,-1];
    zK=2*eK-Mod[-Re[z],2*eK]+I*Im[z];
    pK=Ceiling[Re[z]/(2*eK)-1];
    pmN^pK*LameES[v,n,{zK,k}]]/;
  (Not[lameSPolyQ[v,n]] && Re[z]>N[2*EllipticK[k]] && Precision[z]<Infinity)

LameES[v_,n_Integer?Positive,{z_,k_}]:=
  Module[{eK,pmN,zK,pK},
    eK=N[EllipticK[k]];
    pmN=If[EvenQ[n],1,-1];
    zK=Mod[Re[z],2*eK]+I*Im[z];
    pK=-Floor[Re[z]/(2*eK)];
    pmN^pK*LameES[v,n,{zK,k}]]/;
  (Not[lameSPolyQ[v,n]] && Re[z]<0 && Precision[z]<Infinity)

(* Polynomials case *)
LameES[v_,n_Integer?Positive,{z_,k_}]:=
  If[EvenQ[n],
    If[EvenQ[Round[v]],
      LamePolynomialSCE[v,n/2-1,{z,k}],
      LamePolynomialSCDE[v,n/2-1,{z,k}]],
    If[EvenQ[Round[v]],
      LamePolynomialCDE[v,(n-1)/2,{z,k}],
      LamePolynomialCE[v,(n-1)/2,{z,k}]]]/;
  (lameSPolyQ[v,n] && Precision[z]<Infinity)

(*------------- Derivative of Lame function Ec'[v,n][z,k] -------------*)
LameECPrime[v_,n_Integer?NonNegative,{z_,k_}]:=
  Module[{w0,w1},
    w0=initECK[v,n,k];
    w1=initECPrimeK[v,n,k];
    If[Abs[Im[z]]>0,
      LamePrimeK[leA[n,v,k],v,{z,k},{w0,w1}],
      initLameECPrime[v,n,{z,k}]]]/;
  (Not[lameCPolyQ[v,n]] && 0<=Re[z]<=N[2*EllipticK[k]] && Precision[z]<Infinity)

LameECPrime[v_,n_Integer?NonNegative,{z_,k_}]:=
  Module[{eK,pmN,zK,pK},
    eK=N[EllipticK[k]];
    pmN=If[EvenQ[n],1,-1];
    zK=2*eK-Mod[-Re[z],2*eK]+I*Im[z];
    pK=Ceiling[Re[z]/(2*eK)-1];
    pmN^pK*LameECPrime[v,n,{zK,k}]]/;
  (Not[lameCPolyQ[v,n]] && Re[z]>N[2*EllipticK[k]] && Precision[z]<Infinity)

LameECPrime[v_,n_Integer?NonNegative,{z_,k_}]:=
  Module[{eK,pmN,zK,pK},
    eK=N[EllipticK[k]];
    pmN=If[EvenQ[n],1,-1];
    zK=Mod[Re[z],2*eK]+I*Im[z];
    pK=-Floor[Re[z]/(2*eK)];
    pmN^pK*LameECPrime[v,n,{zK,k}]]/;
  (Not[lameCPolyQ[v,n]] && Re[z]<0 && Precision[z]<Infinity)

(* Polynomials case *)
LameECPrime[v_,n_Integer?NonNegative,{z_,k_}]:=
  If[EvenQ[n],
    If[EvenQ[Round[v]],
      LamePolynomialUEPrime[v,n/2,{z,k}],
      LamePolynomialDEPrime[v,n/2,{z,k}]],
    If[EvenQ[Round[v]],
      LamePolynomialSDEPrime[v,(n-1)/2,{z,k}],
      LamePolynomialSEPrime[v,(n-1)/2,{z,k}]]]/;
  (lameCPolyQ[v,n] && Precision[z]<Infinity)

(*------------- Derivative of Lame function Es'[v,n][z,k] -------------*)
LameESPrime[v_,n_Integer?Positive,{z_,k_}]:=
  Module[{w0,w1},
    w0=initESK[v,n,k];
    w1=initESPrimeK[v,n,k];
    If[Abs[Im[z]]>0,
      LamePrimeK[leB[n,v,k],v,{z,k},{w0,w1}],
      initLameESPrime[v,n,{z,k}]]]/;
  (Not[lameSPolyQ[v,n]] && 0<=Re[z]<=N[2*EllipticK[k]] && Precision[z]<Infinity)

LameESPrime[v_,n_Integer?Positive,{z_,k_}]:=
  Module[{eK,pmN,zK,pK},
    eK=N[EllipticK[k]];
    pmN=If[EvenQ[n],1,-1];
    zK=2*eK-Mod[-Re[z],2*eK]+I*Im[z];
    pK=Ceiling[Re[z]/(2*eK)-1];
    pmN^pK*LameESPrime[v,n,{zK,k}]]/;
  (Not[lameSPolyQ[v,n]] && Re[z]>N[2*EllipticK[k]] && Precision[z]<Infinity)

LameESPrime[v_,n_Integer?Positive,{z_,k_}]:=
  Module[{eK,pmN,zK,pK},
    eK=N[EllipticK[k]];
    pmN=If[EvenQ[n],1,-1];
    zK=Mod[Re[z],2*eK]+I*Im[z];
    pK=-Floor[Re[z]/(2*eK)];
    pmN^pK*LameESPrime[v,n,{zK,k}]]/;
  (Not[lameSPolyQ[v,n]] && Re[z]<0 && Precision[z]<Infinity)

(* Polynomials case *)
LameESPrime[v_,n_Integer?Positive,{z_,k_}]:=
  If[EvenQ[n],
    If[EvenQ[Round[v]],
      LamePolynomialSCEPrime[v,n/2-1,{z,k}],
      LamePolynomialSCDEPrime[v,n/2-1,{z,k}]],
    If[EvenQ[Round[v]],
      LamePolynomialCDEPrime[v,(n-1)/2,{z,k}],
      LamePolynomialCEPrime[v,(n-1)/2,{z,k}]]]/;
  (lameSPolyQ[v,n] && Precision[z]<Infinity)

(*---------------- Initial values of Lame function Fc -----------------*)
initLameFC[v_,n_Integer?NonNegative,{z_,k_}]:=
  Module[{w1},
    w1=(n+1)/initEC0[v,n,k];
    If[Abs[Im[z]]>0,
      Lame[leA[n,v,k],v,{z,k},{0,w1}],
      Re[Lame[leA[n,v,k],v,{z,k},{0,w1}]]]]/;
  (EvenQ[n] && Precision[z]<Infinity)

initLameFC[v_,n_Integer?NonNegative,{z_,k_}]:=
  Module[{w0},
    w0=-(n+1)/initECPrime0[v,n,k];
    If[Abs[Im[z]]>0,
      Lame[leA[n,v,k],v,{z,k},{w0,0}],
      Re[Lame[leA[n,v,k],v,{z,k},{w0,0}]]]]/;
  (OddQ[n] && Precision[z]<Infinity)

(*---------------- Initial values of Lame function Fs -----------------*)
initLameFS[v_,n_Integer?Positive,{z_,k_}]:=
  Module[{w0},
    w0=-(n+1)/initESPrime0[v,n,k];
    If[Abs[Im[z]]>0,
      Lame[leB[n,v,k],v,{z,k},{w0,0}],
      Re[Lame[leB[n,v,k],v,{z,k},{w0,0}]]]]/;
  (EvenQ[n] && Precision[z]<Infinity)

initLameFS[v_,n_Integer?Positive,{z_,k_}]:=
  Module[{w1},
    w1=(n+1)/initES0[v,n,k];
    If[Abs[Im[z]]>0,
      Lame[leB[n,v,k],v,{z,k},{0,w1}],
      Re[Lame[leB[n,v,k],v,{z,k},{0,w1}]]]]/;
  (OddQ[n] && Precision[z]<Infinity)

(*----------- Initial values of Derivative Lame function Fc -----------*)
initLameFCPrime[v_,n_Integer?NonNegative,{z_,k_}]:=
  Module[{w1},
    w1=(n+1)/initEC0[v,n,k];
    If[Abs[Im[z]]>0,
      LamePrime[leA[n,v,k],v,{z,k},{0,w1}],
      Re[LamePrime[leA[n,v,k],v,{z,k},{0,w1}]]]]/;
  (EvenQ[n] && Precision[z]<Infinity)

initLameFCPrime[v_,n_Integer?NonNegative,{z_,k_}]:=
  Module[{w0},
    w0=-(n+1)/initECPrime0[v,n,k];
    If[Abs[Im[z]]>0,
      LamePrime[leA[n,v,k],v,{z,k},{w0,0}],
      Re[LamePrime[leA[n,v,k],v,{z,k},{w0,0}]]]]/;
  (OddQ[n] && Precision[z]<Infinity)

(*----------- Initial values of Derivative Lame function Fs -----------*)
initLameFSPrime[v_,n_Integer?Positive,{z_,k_}]:=
  Module[{w0},
    w0=-(n+1)/initESPrime0[v,n,k];
    If[Abs[Im[z]]>0,
      LamePrime[leB[n,v,k],v,{z,k},{w0,0}],
      Re[LamePrime[leB[n,v,k],v,{z,k},{w0,0}]]]]/;
  (EvenQ[n] && Precision[z]<Infinity)

initLameFSPrime[v_,n_Integer?Positive,{z_,k_}]:=
  Module[{w1},
    w1=(n+1)/initES0[v,n,k];
    If[Abs[Im[z]]>0,
      LamePrime[leB[n,v,k],v,{z,k},{0,w1}],
      Re[LamePrime[leB[n,v,k],v,{z,k},{0,w1}]]]]/;
  (OddQ[n] && Precision[z]<Infinity)

(*-------------------- Lame function Fc[v,n][z,k] ---------------------*)
(* Complex variables "z" *)
LameFC[v_,n_Integer?NonNegative,{z_,k_}]:=
  Module[{w0,w1},
    w0=initFCK[v,n,k];
    w1=initFCPrimeK[v,n,k];
    LameK[leA[n,v,k],v,{z,k},{w0,w1}]]/;
  (Abs[Im[z]]>0 && 0<=Re[z]<=N[2*EllipticK[k]] && Precision[z]<Infinity)

LameFC[v_,n_Integer?NonNegative,{z_,k_}]:=
  Module[{eK,cons,pmN,zK,pK},
    eK=N[EllipticK[k]];
    pmN=If[EvenQ[n],1,-1];
    cons=2*initFCK[v,n,k]/initECK[v,n,k];
    zK=2*eK-Mod[-Re[z],2*eK]+I*Im[z];
    pK=Ceiling[Re[z]/(2*eK)-1];
    pmN^pK*(LameFC[v,n,{zK,k}]+pK*cons*LameEC[v,n,{zK,k}])]/;
  (Abs[Im[z]]>0 && Re[z]>N[2*EllipticK[k]] && Precision[z]<Infinity)

LameFC[v_,n_Integer?NonNegative,{z_,k_}]:=
  Module[{eK,cons,pmN,zK,pK},
    eK=N[EllipticK[k]];
    pmN=If[EvenQ[n],1,-1];
    cons=2*initFCK[v,n,k]/initECK[v,n,k];
    zK=Mod[Re[z],2*eK]+I*Im[z];
    pK=-Floor[Re[z]/(2*eK)];
    pmN^pK*(LameFC[v,n,{zK,k}]-pK*cons*LameEC[v,n,{zK,k}])]/;
  (Abs[Im[z]]>0 && Re[z]<0 && Precision[z]<Infinity)

(* Real variables "z" *)
LameFC[v_,n_Integer?NonNegative,{z_,k_}]:=
  initLameFC[v,n,{z,k}]/;
  (N[Im[z]]===0. && Abs[Re[z]]<=N[EllipticK[k]] && Precision[z]<Infinity)

LameFC[v_,n_Integer?NonNegative,{z_,k_}]:=
  Module[{eK,cons,pmN,zK,pK},
    eK=N[EllipticK[k]];
    pmN=If[EvenQ[n],1,-1];
    cons=2*initFCK[v,n,k]/initECK[v,n,k];
    zK=eK-Mod[-Re[z]-eK,2*eK]+I*Im[z];
    pK=Ceiling[Re[z]/(2*eK)-1/2];
    pmN^pK*(LameFC[v,n,{zK,k}]+pK*cons*LameEC[v,n,{zK,k}])]/;
  (N[Im[z]]===0. && Re[z]>N[EllipticK[k]] && Precision[z]<Infinity)

LameFC[v_,n_Integer?NonNegative,{z_,k_}]:=
  Module[{eK,cons,pmN,zK,pK},
    eK=N[EllipticK[k]];
    pmN=If[EvenQ[n],1,-1];
    cons=2*initFCK[v,n,k]/initECK[v,n,k];
    zK=Mod[Re[z]+eK,2*eK]-eK+I*Im[z];
    pK=-Floor[Re[z]/(2*eK)+1/2];
    pmN^pK*(LameFC[v,n,{zK,k}]-pK*cons*LameEC[v,n,{zK,k}])]/;
  (N[Im[z]]===0. && Re[z]<-N[EllipticK[k]] && Precision[z]<Infinity)

(*-------------------- Lame function Fs[v,n][z,k] ---------------------*)
(* Complex variables "z" *)
LameFS[v_,n_Integer?Positive,{z_,k_}]:=
  Module[{w0,w1},
    w0=initFSK[v,n,k];
    w1=initFSPrimeK[v,n,k];
    LameK[leB[n,v,k],v,{z,k},{w0,w1}]]/;
  (Abs[Im[z]]>0 && 0<=Re[z]<=N[2*EllipticK[k]] && Precision[z]<Infinity)

LameFS[v_,n_Integer?Positive,{z_,k_}]:=
  Module[{eK,cons,pmN,zK,pK},
    eK=N[EllipticK[k]];
    pmN=If[EvenQ[n],1,-1];
    cons=2*initFSPrimeK[v,n,k]/initESPrimeK[v,n,k];
    zK=2*eK-Mod[-Re[z],2*eK]+I*Im[z];
    pK=Ceiling[Re[z]/(2*eK)-1];
    pmN^pK*(LameFS[v,n,{zK,k}]+pK*cons*LameES[v,n,{zK,k}])]/;
  (Abs[Im[z]]>0 && Re[z]>N[2*EllipticK[k]] && Precision[z]<Infinity)

LameFS[v_,n_Integer?Positive,{z_,k_}]:=
  Module[{eK,cons,pmN,zK,pK},
    eK=N[EllipticK[k]];
    pmN=If[EvenQ[n],1,-1];
    cons=2*initFSPrimeK[v,n,k]/initESPrimeK[v,n,k];
    zK=Mod[Re[z],2*eK]+I*Im[z];
    pK=-Floor[Re[z]/(2*eK)];
    pmN^pK*(LameFS[v,n,{zK,k}]-pK*cons*LameES[v,n,{zK,k}])]/;
  (Abs[Im[z]]>0 && Re[z]<0 && Precision[z]<Infinity)

(* Real variables "z" *)
LameFS[v_,n_Integer?Positive,{z_,k_}]:=
  initLameFS[v,n,{z,k}]/;
  (N[Im[z]]===0. && Abs[Re[z]]<=N[EllipticK[k]] && Precision[z]<Infinity)

LameFS[v_,n_Integer?Positive,{z_,k_}]:=
  Module[{eK,cons,pmN,zK,pK},
    eK=N[EllipticK[k]];
    pmN=If[EvenQ[n],1,-1];
    cons=2*initFSPrimeK[v,n,k]/initESPrimeK[v,n,k];
    zK=eK-Mod[-Re[z]-eK,2*eK]+I*Im[z];
    pK=Ceiling[Re[z]/(2*eK)-1/2];
    pmN^pK*(LameFS[v,n,{zK,k}]+pK*cons*LameES[v,n,{zK,k}])]/;
  (N[Im[z]]===0. && Re[z]>N[EllipticK[k]] && Precision[z]<Infinity)

LameFS[v_,n_Integer?Positive,{z_,k_}]:=
  Module[{eK,cons,pmN,zK,pK},
    eK=N[EllipticK[k]];
    pmN=If[EvenQ[n],1,-1];
    cons=2*initFSPrimeK[v,n,k]/initESPrimeK[v,n,k];
    zK=Mod[Re[z]+eK,2*eK]-eK+I*Im[z];
    pK=-Floor[Re[z]/(2*eK)+1/2];
    pmN^pK*(LameFS[v,n,{zK,k}]-pK*cons*LameES[v,n,{zK,k}])]/;
  (N[Im[z]]===0. && Re[z]<-N[EllipticK[k]] && Precision[z]<Infinity)

(*------------- Derivative of Lame function Fc'[v,n][z,k] -------------*)
(* Complex variables "z" *)
LameFCPrime[v_,n_Integer?NonNegative,{z_,k_}]:=
  Module[{w0,w1},
    w0=initFCK[v,n,k];
    w1=initFCPrimeK[v,n,k];
    LamePrimeK[leA[n,v,k],v,{z,k},{w0,w1}]]/;
  (Abs[Im[z]]>0 && 0<=Re[z]<=N[2*EllipticK[k]] && Precision[z]<Infinity)

LameFCPrime[v_,n_Integer?NonNegative,{z_,k_}]:=
  Module[{eK,cons,pmN,zK,pK},
    eK=N[EllipticK[k]];
    pmN=If[EvenQ[n],1,-1];
    cons=2*initFCK[v,n,k]/initECK[v,n,k];
    zK=2*eK-Mod[-Re[z],2*eK]+I*Im[z];
    pK=Ceiling[Re[z]/(2*eK)-1];
    pmN^pK*(LameFCPrime[v,n,{zK,k}]+pK*cons*LameECPrime[v,n,{zK,k}])]/;
  (Abs[Im[z]]>0 && Re[z]>N[2*EllipticK[k]] && Precision[z]<Infinity)

LameFCPrime[v_,n_Integer?NonNegative,{z_,k_}]:=
  Module[{eK,cons,pmN,zK,pK},
    eK=N[EllipticK[k]];
    pmN=If[EvenQ[n],1,-1];
    cons=2*initFCK[v,n,k]/initECK[v,n,k];
    zK=Mod[Re[z],2*eK]+I*Im[z];
    pK=-Floor[Re[z]/(2*eK)];
    pmN^pK*(LameFCPrime[v,n,{zK,k}]-pK*cons*LameECPrime[v,n,{zK,k}])]/;
  (Abs[Im[z]]>0 && Re[z]<0 && Precision[z]<Infinity)

(* Real variables "z" *)
LameFCPrime[v_,n_Integer?NonNegative,{z_,k_}]:=
  initLameFCPrime[v,n,{z,k}]/;
  (N[Im[z]]===0. && Abs[Re[z]]<=N[EllipticK[k]] && Precision[z]<Infinity)

LameFCPrime[v_,n_Integer?NonNegative,{z_,k_}]:=
  Module[{eK,cons,pmN,zK,pK},
    eK=N[EllipticK[k]];
    pmN=If[EvenQ[n],1,-1];
    cons=2*initFCK[v,n,k]/initECK[v,n,k];
    zK=eK-Mod[-Re[z]-eK,2*eK]+I*Im[z];
    pK=Ceiling[Re[z]/(2*eK)-1/2];
    pmN^pK*(LameFCPrime[v,n,{zK,k}]+pK*cons*LameECPrime[v,n,{zK,k}])]/;
  (N[Im[z]]===0. && Re[z]>N[EllipticK[k]] && Precision[z]<Infinity)

LameFCPrime[v_,n_Integer?NonNegative,{z_,k_}]:=
  Module[{eK,cons,pmN,zK,pK},
    eK=N[EllipticK[k]];
    pmN=If[EvenQ[n],1,-1];
    cons=2*initFCK[v,n,k]/initECK[v,n,k];
    zK=Mod[Re[z]+eK,2*eK]-eK+I*Im[z];
    pK=-Floor[Re[z]/(2*eK)+1/2];
    pmN^pK*(LameFCPrime[v,n,{zK,k}]-pK*cons*LameECPrime[v,n,{zK,k}])]/;
  (N[Im[z]]===0. && Re[z]<-N[EllipticK[k]] && Precision[z]<Infinity)

(*------------- Derivative of Lame function Fs'[v,n][z,k] -------------*)
(* Complex variables "z" *)
LameFSPrime[v_,n_Integer?Positive,{z_,k_}]:=
  Module[{w0,w1},
    w0=initFSK[v,n,k];
    w1=initFSPrimeK[v,n,k];
    LamePrimeK[leB[n,v,k],v,{z,k},{w0,w1}]]/;
  (Abs[Im[z]]>0 && 0<=Re[z]<=N[2*EllipticK[k]] && Precision[z]<Infinity)

LameFSPrime[v_,n_Integer?Positive,{z_,k_}]:=
  Module[{eK,cons,pmN,zK,pK},
    eK=N[EllipticK[k]];
    pmN=If[EvenQ[n],1,-1];
    cons=2*initFSPrimeK[v,n,k]/initESPrimeK[v,n,k];
    zK=2*eK-Mod[-Re[z],2*eK]+I*Im[z];
    pK=Ceiling[Re[z]/(2*eK)-1];
    pmN^pK*(LameFSPrime[v,n,{zK,k}]+pK*cons*LameESPrime[v,n,{zK,k}])]/;
  (Abs[Im[z]]>0 && Re[z]>N[2*EllipticK[k]] && Precision[z]<Infinity)

LameFSPrime[v_,n_Integer?Positive,{z_,k_}]:=
  Module[{eK,cons,pmN,zK,pK},
    eK=N[EllipticK[k]];
    pmN=If[EvenQ[n],1,-1];
    cons=2*initFSPrimeK[v,n,k]/initESPrimeK[v,n,k];
    zK=Mod[Re[z],2*eK]+I*Im[z];
    pK=-Floor[Re[z]/(2*eK)];
    pmN^pK*(LameFSPrime[v,n,{zK,k}]-pK*cons*LameESPrime[v,n,{zK,k}])]/;
  (Abs[Im[z]]>0 && Re[z]<0 && Precision[z]<Infinity)

(* Real variables "z" *)
LameFSPrime[v_,n_Integer?Positive,{z_,k_}]:=
  initLameFSPrime[v,n,{z,k}]/;
  (N[Im[z]]===0. && Abs[Re[z]]<=N[EllipticK[k]] && Precision[z]<Infinity)

LameFSPrime[v_,n_Integer?Positive,{z_,k_}]:=
  Module[{eK,cons,pmN,zK,pK},
    eK=N[EllipticK[k]];
    pmN=If[EvenQ[n],1,-1];
    cons=2*initFSPrimeK[v,n,k]/initESPrimeK[v,n,k];
    zK=eK-Mod[-Re[z]-eK,2*eK]+I*Im[z];
    pK=Ceiling[Re[z]/(2*eK)-1/2];
    pmN^pK*(LameFSPrime[v,n,{zK,k}]+pK*cons*LameESPrime[v,n,{zK,k}])]/;
  (N[Im[z]]===0. && Re[z]>N[EllipticK[k]] && Precision[z]<Infinity)

LameFSPrime[v_,n_Integer?Positive,{z_,k_}]:=
  Module[{eK,cons,pmN,zK,pK},
    eK=N[EllipticK[k]];
    pmN=If[EvenQ[n],1,-1];
    cons=2*initFSPrimeK[v,n,k]/initESPrimeK[v,n,k];
    zK=Mod[Re[z]+eK,2*eK]-eK+I*Im[z];
    pK=-Floor[Re[z]/(2*eK)+1/2];
    pmN^pK*(LameFSPrime[v,n,{zK,k}]-pK*cons*LameESPrime[v,n,{zK,k}])]/;
  (N[Im[z]]===0. && Re[z]<-N[EllipticK[k]] && Precision[z]<Infinity)

(*-------------------- Lame function Lc[v,n][z,k] ---------------------*)

LameLC[v_,n_Integer?NonNegative,{z_,k_}]:=
  Module[{w0,w1},
    w0=initECK[v,n,k]+I*initFCK[v,n,k];
    w1=initECPrimeK[v,n,k]+I*initFCPrimeK[v,n,k];
    LameK[leA[n,v,k],v,{z,k},{w0,w1}]]/;
  (0<=Re[z]<=N[2*EllipticK[k]] && Precision[z]<Infinity)

LameLC[v_,n_Integer?NonNegative,{z_,k_}]:=
  Module[{eK,cons,pmN,zK,pK},
    eK=N[EllipticK[k]];
    pmN=If[EvenQ[n],1,-1];
    cons=2*I*initFCK[v,n,k]/initECK[v,n,k];
    zK=2*eK-Mod[-Re[z],2*eK]+I*Im[z];
    pK=Ceiling[Re[z]/(2*eK)-1];
    pmN^pK*(LameLC[v,n,{zK,k}]+pK*cons*LameEC[v,n,{zK,k}])]/;
  (Re[z]>N[2*EllipticK[k]] && Precision[z]<Infinity)

LameLC[v_,n_Integer?NonNegative,{z_,k_}]:=
  Module[{eK,cons,pmN,zK,pK},
    eK=N[EllipticK[k]];
    pmN=If[EvenQ[n],1,-1];
    cons=2*I*initFCK[v,n,k]/initECK[v,n,k];
    zK=Mod[Re[z],2*eK]+I*Im[z];
    pK=-Floor[Re[z]/(2*eK)];
    pmN^pK*(LameLC[v,n,{zK,k}]-pK*cons*LameEC[v,n,{zK,k}])]/;
  (Re[z]<0 && Precision[z]<Infinity)

(*-------------------- Lame function Ls[v,n][z,k] ---------------------*)

LameLS[v_,n_Integer?Positive,{z_,k_}]:=
  Module[{w0,w1},
    w0=initESK[v,n,k]+I*initFSK[v,n,k];
    w1=initESPrimeK[v,n,k]+I*initFSPrimeK[v,n,k];
    LameK[leB[n,v,k],v,{z,k},{w0,w1}]]/;
  (0<=Re[z]<=N[2*EllipticK[k]] && Precision[z]<Infinity)

LameLS[v_,n_Integer?Positive,{z_,k_}]:=
  Module[{eK,cons,pmN,zK,pK},
    eK=N[EllipticK[k]];
    pmN=If[EvenQ[n],1,-1];
    cons=2*I*initFSPrimeK[v,n,k]/initESPrimeK[v,n,k];
    zK=2*eK-Mod[-Re[z],2*eK]+I*Im[z];
    pK=Ceiling[Re[z]/(2*eK)-1];
    pmN^pK*(LameLS[v,n,{zK,k}]+pK*cons*LameES[v,n,{zK,k}])]/;
  (Re[z]>N[2*EllipticK[k]] && Precision[z]<Infinity)

LameLS[v_,n_Integer?Positive,{z_,k_}]:=
  Module[{eK,cons,pmN,zK,pK},
    eK=N[EllipticK[k]];
    pmN=If[EvenQ[n],1,-1];
    cons=2*I*initFSPrimeK[v,n,k]/initESPrimeK[v,n,k];
    zK=Mod[Re[z],2*eK]+I*Im[z];
    pK=-Floor[Re[z]/(2*eK)];
    pmN^pK*(LameLS[v,n,{zK,k}]-pK*cons*LameES[v,n,{zK,k}])]/;
  (Re[z]<0 && Precision[z]<Infinity)

(*------------- Derivative of Lame function Lc'[v,n][z,k] -------------*)

LameLCPrime[v_,n_Integer?NonNegative,{z_,k_}]:=
  Module[{w0,w1},
    w0=initECK[v,n,k]+I*initFCK[v,n,k];
    w1=initECPrimeK[v,n,k]+I*initFCPrimeK[v,n,k];
    LamePrimeK[leA[n,v,k],v,{z,k},{w0,w1}]]/;
  (0<=Re[z]<=N[2*EllipticK[k]] && Precision[z]<Infinity)

LameLCPrime[v_,n_Integer?NonNegative,{z_,k_}]:=
  Module[{eK,cons,pmN,zK,pK},
    eK=N[EllipticK[k]];
    pmN=If[EvenQ[n],1,-1];
    cons=2*I*initFCK[v,n,k]/initECK[v,n,k];
    zK=2*eK-Mod[-Re[z],2*eK]+I*Im[z];
    pK=Ceiling[Re[z]/(2*eK)-1];
    pmN^pK*(LameLCPrime[v,n,{zK,k}]+pK*cons*LameECPrime[v,n,{zK,k}])]/;
  (Re[z]>N[2*EllipticK[k]] && Precision[z]<Infinity)

LameLCPrime[v_,n_Integer?NonNegative,{z_,k_}]:=
  Module[{eK,cons,pmN,zK,pK},
    eK=N[EllipticK[k]];
    pmN=If[EvenQ[n],1,-1];
    cons=2*I*initFCK[v,n,k]/initECK[v,n,k];
    zK=Mod[Re[z],2*eK]+I*Im[z];
    pK=-Floor[Re[z]/(2*eK)];
    pmN^pK*(LameLCPrime[v,n,{zK,k}]-pK*cons*LameECPrime[v,n,{zK,k}])]/;
  (Re[z]<0 && Precision[z]<Infinity)

(*------------- Derivative of Lame function Ls'[v,n][z,k] -------------*)

LameLSPrime[v_,n_Integer?Positive,{z_,k_}]:=
  Module[{w0,w1},
    w0=initESK[v,n,k]+I*initFSK[v,n,k];
    w1=initESPrimeK[v,n,k]+I*initFSPrimeK[v,n,k];
    LamePrimeK[leB[n,v,k],v,{z,k},{w0,w1}]]/;
  (0<=Re[z]<=N[2*EllipticK[k]] && Precision[z]<Infinity)

LameLSPrime[v_,n_Integer?Positive,{z_,k_}]:=
  Module[{eK,cons,pmN,zK,pK},
    eK=N[EllipticK[k]];
    pmN=If[EvenQ[n],1,-1];
    cons=2*I*initFSPrimeK[v,n,k]/initESPrimeK[v,n,k];
    zK=2*eK-Mod[-Re[z],2*eK]+I*Im[z];
    pK=Ceiling[Re[z]/(2*eK)-1];
    pmN^pK*(LameLSPrime[v,n,{zK,k}]+pK*cons*LameESPrime[v,n,{zK,k}])]/;
  (Re[z]>N[2*EllipticK[k]] && Precision[z]<Infinity)

LameLSPrime[v_,n_Integer?Positive,{z_,k_}]:=
  Module[{eK,cons,pmN,zK,pK},
    eK=N[EllipticK[k]];
    pmN=If[EvenQ[n],1,-1];
    cons=2*I*initFSPrimeK[v,n,k]/initESPrimeK[v,n,k];
    zK=Mod[Re[z],2*eK]+I*Im[z];
    pK=-Floor[Re[z]/(2*eK)];
    pmN^pK*(LameLSPrime[v,n,{zK,k}]-pK*cons*LameESPrime[v,n,{zK,k}])]/;
  (Re[z]<0 && Precision[z]<Infinity)



(**********************************************************************************)
(*                                Lame polynomials                                *)
(**********************************************************************************)

(*---------------- Coefficients of recurrence relation ----------------*)

alpPCe[p_,v_,k_]:=
  Which[N[p]===0.,v (v+1) k,p>0,(v-2 p) (v+2 p+1) k/2];
betPCe[p_,v_,k_]:=4*p^2*(2-k);
gamPCe[p_,v_,k_]:=(v-2 p+1) (v+2 p) k/2;

alpPCo[p_,v_,k_]:=(v-2 p-1) (v+2 p+2) k/2;
betPCo[p_,v_,k_]:=
  Which[N[p]===0.,2-k+v (v+1) k/2,p>0,(2 p+1)^2*(2-k)];
gamPCo[p_,v_,k_]:=(v-2 p) (v+2 p+1) k/2;

alpPDe[p_,v_,k_]:=(v-2 p-2) (v+2 p+3) k/2;
betPDe[p_,v_,k_]:=(2 p+2)^2*(2-k);
gamPDe[p_,v_,k_]:=(v-2 p-1) (v+2 p+2) k/2;

alpPDo[p_,v_,k_]:=(v-2 p-1) (v+2 p+2) k/2;
betPDo[p_,v_,k_]:=
  Which[N[p]===0.,2-k-v (v+1) k/2,p>0,(2 p+1)^2*(2-k)];
gamPDo[p_,v_,k_]:=(v-2 p) (v+2 p+1) k/2;

(*----------------- Coefficients of Fourier series C ------------------*)

(* Coefficients Ce[n,v,k][m] *)
coeffLTe[n_,v_,k_,0]:=
  (leHC[n,v,k]-betPCe[0,v,k])/alpPCe[0,v,k];

coeffLTe[n_,v_,k_,m_/;(EvenQ[m] && m>0)]:=
  Module[{clte=coeffLTe[n,v,k,m-2]},
    If[N[clte]===0.,0,
      (leHC[n,v,k]-betPCe[m/2,v,k]-gamPCe[m/2,v,k]/clte)/alpPCe[m/2,v,k]]];

coeffLTe[n_,v_,k_,m_/;Not[EvenQ[m] && m>=0]]:=0;

coeffLameTe[n_,v_,k_,m_]:=
  coeffLameTe[n,v,k,m]=coeffLTe[n,v,k,m];

coeffLameCe[n_,v_,k_,0]:=
  1/Sqrt[(1-k/2)/2+coeffLameTe[n,v,k,0]*((1-k/2)*coeffLameTe[n,v,k,0]-k/2)+
    Sum[Product[coeffLameTe[n,v,k,i]^2,{i,0,2 h-4,2}]*coeffLameTe[n,v,k,2 h-2]*
      ((1-k/2)*coeffLameTe[n,v,k,2 h-2]-k/2),{h,2,(v-1)/2}]];

coeffLameCe[n_,v_,k_,m_/;(EvenQ[m] && m>0)]:=
  Product[coeffLameTe[n,v,k,h],{h,0,m-2,2}]*coeffLameCe[n,v,k,0];

coeffLameCe[n_,v_,k_,m_/;Not[EvenQ[m] && m>=0]]:=0;

(* Coefficients Co[n,v,k][m] *)
coeffLTo[n_,v_,k_,1]:=
  (leHC[n,v,k]-betPCo[0,v,k])/alpPCo[0,v,k];

coeffLTo[n_,v_,k_,m_/;(OddQ[m] && m>1)]:=
  Module[{clto=coeffLTo[n,v,k,m-2]},
    If[N[clto]===0.,0,
      (leHC[n,v,k]-betPCo[(m-1)/2,v,k]-gamPCo[(m-1)/2,v,k]/clto)/alpPCo[(m-1)/2,v,k]]];

coeffLTo[n_,v_,k_,m_/;Not[OddQ[m] && m>=1]]:=0;

coeffLameTo[n_,v_,k_,m_]:=
  coeffLameTo[n,v,k,m]=coeffLTo[n,v,k,m];

coeffLameCo[n_,v_,k_,1]:=
  1/Sqrt[1-3 k/4+coeffLameTo[n,v,k,1]*((1-k/2)*coeffLameTo[n,v,k,1]-k/2)+
    Sum[Product[coeffLameTo[n,v,k,i]^2,{i,1,2 h-3,2}]*coeffLameTo[n,v,k,2 h-1]*
      ((1-k/2)*coeffLameTo[n,v,k,2 h-1]-k/2),{h,2,(v-2)/2}]];

coeffLameCo[n_,v_,k_,m_/;(OddQ[m] && m>1)]:=
  Product[coeffLameTo[n,v,k,h],{h,1,m-2,2}]*coeffLameCo[n,v,k,1];

coeffLameCo[n_,v_,k_,m_/;Not[OddQ[m] && m>=1]]:=0;

(* Coefficients C[n,v,k][m] *)
LameCoefficientC[n_Integer,v_,k_,m_Integer]:=
  LameCoefficientC[n,v,k,m]=
    Which[EvenQ[n] && EvenQ[m],coeffLameCe[n,v,k,m],
      OddQ[n] && OddQ[m],coeffLameCo[n,v,k,m]];

(*----------------- Coefficients of Fourier series D ------------------*)

(* Coefficients De[n,v,k][m] *)
coeffLUe[n_,v_,k_,2]:=
  (leHS[n,v,k]-betPDe[0,v,k])/alpPDe[0,v,k];

coeffLUe[n_,v_,k_,m_/;(EvenQ[m] && m>2)]:=
  Module[{clue=coeffLUe[n,v,k,m-2]},
    If[N[clue]===0.,0,
    (leHS[n,v,k]-betPDe[(m-2)/2,v,k]-gamPDe[(m-2)/2,v,k]/clue)/alpPDe[(m-2)/2,v,k]]];

coeffLUe[n_,v_,k_,m_/;Not[EvenQ[m] && m>=2]]:=0;

coeffLameUe[n_,v_,k_,m_]:=
  coeffLameUe[n,v,k,m]=coeffLUe[n,v,k,m];

coeffLameDe[n_,v_,k_,2]:=
  1/Sqrt[(1-k/2)+coeffLameUe[n,v,k,2]*((1-k/2)*coeffLameUe[n,v,k,2]-k/2)+
    Sum[Product[coeffLameUe[n,v,k,i]^2,{i,2,2 h-2,2}]*coeffLameUe[n,v,k,2 h]*
      ((1-k/2)*coeffLameUe[n,v,k,2 h]-k/2),{h,2,(v-3)/2}]];

coeffLameDe[n_,v_,k_,m_/;(EvenQ[m] && m>2)]:=
  Product[coeffLameUe[n,v,k,h],{h,2,m-2,2}]*coeffLameDe[n,v,k,2];

coeffLameDe[n_,v_,k_,m_/;Not[EvenQ[m] && m>=2]]:=0;

(* Coefficients Do[n,v,k][m] *)
coeffLUo[n_,v_,k_,1]:=
  (leHS[n,v,k]-betPDo[0,v,k])/alpPDo[0,v,k];

coeffLUo[n_,v_,k_,m_/;(OddQ[m] && m>1)]:=
  Module[{cluo=coeffLUo[n,v,k,m-2]},
    If[N[cluo]===0.,0,
      (leHS[n,v,k]-betPDo[(m-1)/2,v,k]-gamPDo[(m-1)/2,v,k]/cluo)/alpPDo[(m-1)/2,v,k]]];

coeffLUo[n_,v_,k_,m_/;Not[OddQ[m] && m>=1]]:=0;

coeffLameUo[n_,v_,k_,m_]:=
  coeffLameUo[n,v,k,m]=coeffLUo[n,v,k,m];

coeffLameDo[n_,v_,k_,1]:=
  1/Sqrt[1-k/4+coeffLameUo[n,v,k,1]*((1-k/2)*coeffLameUo[n,v,k,1]-k/2)+
    Sum[Product[coeffLameUo[n,v,k,i]^2,{i,1,2 h-3,2}]*coeffLameUo[n,v,k,2 h-1]*
      ((1-k/2)*coeffLameUo[n,v,k,2 h-1]-k/2),{h,2,(v-2)/2}]];

coeffLameDo[n_,v_,k_,m_/;(OddQ[m] && m>1)]:=
  Product[coeffLameUo[n,v,k,h],{h,1,m-2,2}]*coeffLameDo[n,v,k,1];

coeffLameDo[n_,v_,k_,m_/;Not[OddQ[m] && m>=1]]:=0;

(* Coefficients D[n,v,k][m] *)
LameCoefficientD[n_Integer,v_,k_,m_Integer]:=
  LameCoefficientD[n,v,k,m]=
    Which[EvenQ[n] && EvenQ[m],coeffLameDe[n,v,k,m],
      OddQ[n] && OddQ[m],coeffLameDo[n,v,k,m]];

(*-------------------- Jacobi's amplitude am[z,k] ---------------------*)

am[z_,k_]:=
  I*Log[cn[z,k]-I*sn[z,k]]/;
  (-2*EllipticK[k]<=Re[z]<2*EllipticK[k] && Precision[z]<Infinity)

am[z_,k_]:=
  Module[{eK},eK=EllipticK[k];
    am[z-4 eK,k]+2 Pi]/;
  (Re[z]>=2*EllipticK[k] && Precision[z]<Infinity)

am[z_,k_]:=
  Module[{eK},eK=EllipticK[k];
    am[z+4 eK,k]-2 Pi]/;
  (Re[z]<-2*EllipticK[k] && Precision[z]<Infinity)

(* The cn[z,k] is calculate faster than JacobiCN[z,k]. *)
cn[z_,k_]:=
  Module[{q=EllipticNomeQ[k],x=Pi*z/(2*EllipticK[k]),cons},
    cons=EllipticTheta[4,0.,q]/EllipticTheta[2,0.,q];
    cons*EllipticTheta[2,x,q]/EllipticTheta[4,x,q]];

(*-------------------- Lame polynomial function uE --------------------*)

LamePolynomialUE[v_,m_Integer,{z_,k_}]:=
  Module[{vp,zp,kp,phi},
    vp=setPrecLame[v];
    zp=setPrecLame[z];
    kp=setPrecLame[k];
    phi=setPrecLame[Pi/2-am[zp,kp]];
    N[Sum[LameCoefficientA[2 m,vp,kp,2 h]*Cos[(2 h)*phi],{h,1,v/2}]+
      LameCoefficientA[2 m,vp,kp,0]/2]]*sgnFactorPolyU[v,m,k]/;
  (EvenQ[Round[v]] && 0<=m<=v/2 && Precision[z]<Infinity)

(* sgnFactorPolyU[v,n,k] *)
sgnFactorPolyU[v_,m_Integer,k_]:=
  Module[{vp,kp,valUK},
    vp=setPrecLame[v];
    kp=setPrecLame[k];
    valUK=Sum[LameCoefficientA[2 m,vp,kp,2 h],{h,1,v/2}]+
      LameCoefficientA[2 m,vp,kp,0]/2;
    If[Re[valUK]>0,1,-1]];

(*-------------------- Lame polynomial function dE --------------------*)

LamePolynomialDE[v_,m_Integer,{z_,k_}]:=
  Module[{vp,zp,kp,phi},
    vp=setPrecLame[v];
    zp=setPrecLame[z];
    kp=setPrecLame[k];
    phi=setPrecLame[Pi/2-am[zp,kp]];
    N[Sum[LameCoefficientC[2 m,vp,kp,2 h]*Cos[(2 h)*phi],{h,1,(v-1)/2}]+
      LameCoefficientC[2 m,vp,kp,0]/2]]*JacobiDN[z,k]*sgnFactorPolyD[v,m,k]/;
  (OddQ[Round[v]] && 0<=m<=(v-1)/2 && Precision[z]<Infinity)

(* sgnFactorPolyD[v,n,k] *)
sgnFactorPolyD[v_,m_Integer,k_]:=
  Module[{vp,kp,valDK},
    vp=setPrecLame[v];
    kp=setPrecLame[k];
    valDK=Sqrt[1-k]*(Sum[LameCoefficientC[2 m,vp,kp,2 h],{h,1,(v-1)/2}]+
      LameCoefficientC[2 m,vp,kp,0]/2);
    If[Re[valDK]>0,1,-1]];

(*------------------- Lame polynomial function sdE --------------------*)

LamePolynomialSDE[v_,m_Integer,{z_,k_}]:=
  Module[{vp,zp,kp,phi},
    vp=setPrecLame[v];
    zp=setPrecLame[z];
    kp=setPrecLame[k];
    phi=setPrecLame[Pi/2-am[zp,kp]];
    N[Sum[LameCoefficientC[2 m+1,vp,kp,2 h+1]*Cos[(2 h+1)*phi],{h,0,v/2-1}]]]*
      JacobiDN[z,k]*sgnFactorPolySD[v,m,k]/;
  (EvenQ[Round[v]] && 0<=m<=(v/2-1) && Precision[z]<Infinity)

(* sgnFactorPolySD[v,n,k] *)
sgnFactorPolySD[v_,m_Integer,k_]:=
  Module[{vp,kp,valSDK},
    vp=setPrecLame[v];
    kp=setPrecLame[k];
    valSDK=Sqrt[1-k]*Sum[LameCoefficientC[2 m+1,vp,kp,2 h+1],{h,0,v/2-1}];
    If[Re[valSDK]>0,1,-1]];

(*-------------------- Lame polynomial function sE --------------------*)

LamePolynomialSE[v_,m_Integer,{z_,k_}]:=
  Module[{vp,zp,kp,phi},
    vp=setPrecLame[v];
    zp=setPrecLame[z];
    kp=setPrecLame[k];
    phi=setPrecLame[Pi/2-am[zp,kp]];
    N[Sum[LameCoefficientA[2 m+1,vp,kp,2 h+1]*Cos[(2 h+1)*phi],{h,0,(v-1)/2}]]]*
      sgnFactorPolyS[v,m,k]/;
  (OddQ[Round[v]] && 0<=m<=(v-1)/2 && Precision[z]<Infinity)

(* sgnFactorPolyS[v,n,k] *)
sgnFactorPolyS[v_,m_Integer,k_]:=
  Module[{vp,kp,valSK},
    vp=setPrecLame[v];
    kp=setPrecLame[k];
    valSK=Sum[LameCoefficientA[2 m+1,vp,kp,2 h+1],{h,0,(v-1)/2}];
    If[Re[valSK]>0,1,-1]];

(*------------------- Lame polynomial function scE --------------------*)

LamePolynomialSCE[v_,m_Integer,{z_,k_}]:=
  Module[{vp,zp,kp,phi},
    vp=setPrecLame[v];
    zp=setPrecLame[z];
    kp=setPrecLame[k];
    phi=setPrecLame[Pi/2-am[zp,kp]];
    N[Sum[LameCoefficientB[2 m+2,vp,kp,2 h+2]*Sin[(2 h+2)*phi],{h,0,v/2-1}]]]*
      sgnFactorPolySC[v,m,k]/;
  (EvenQ[Round[v]] && 0<=m<=(v/2-1) && Precision[z]<Infinity)

(* sgnFactorPolySC[v,n,k] *)
sgnFactorPolySC[v_,m_Integer,k_]:=
  Module[{vp,kp,valSCK},
    vp=setPrecLame[v];
    kp=setPrecLame[k];
    valSCK=Sqrt[1-k]*Sum[(2 h+2)*LameCoefficientB[2 m+2,vp,kp,2 h+2],{h,0,v/2-1}];
    If[Re[valSCK]>0,1,-1]];

(*------------------- Lame polynomial function scdE -------------------*)

LamePolynomialSCDE[v_,m_Integer,{z_,k_}]:=
  Module[{vp,zp,kp,phi},
    vp=setPrecLame[v];
    zp=setPrecLame[z];
    kp=setPrecLame[k];
    phi=setPrecLame[Pi/2-am[zp,kp]];
    N[Sum[LameCoefficientD[2 m+2,vp,kp,2 h+2]*Sin[(2 h+2)*phi],{h,0,(v-3)/2}]]]*
      JacobiDN[z,k]*sgnFactorPolySCD[v,m,k]/;
  (OddQ[Round[v]] && 0<=m<=(v-3)/2 && Precision[z]<Infinity)

(* sgnFactorPolySCD[v,n,k] *)
sgnFactorPolySCD[v_,m_Integer,k_]:=
  Module[{vp,kp,valSCDK},
    vp=setPrecLame[v];
    kp=setPrecLame[k];
    valSCDK=(1-k)*Sum[(2 h+2)*LameCoefficientD[2 m+2,vp,kp,2 h+2],{h,0,(v-3)/2}];
  If[Re[valSCDK]>0,1,-1]];

(*-------------------- Lame polynomial function cE --------------------*)

LamePolynomialCE[v_,m_Integer,{z_,k_}]:=
  Module[{vp,zp,kp,phi},
    vp=setPrecLame[v];
    zp=setPrecLame[z];
    kp=setPrecLame[k];
    phi=setPrecLame[Pi/2-am[zp,kp]];
    N[Sum[LameCoefficientB[2 m+1,vp,kp,2 h+1]*Sin[(2 h+1)*phi],{h,0,(v-1)/2}]]]*
      sgnFactorPolyC[v,m,k]/;
  (OddQ[Round[v]] && 0<=m<=(v-1)/2 && Precision[z]<Infinity)

(* sgnFactorPolyC[v,n,k] *)
sgnFactorPolyC[v_,m_Integer,k_]:=
  Module[{vp,kp,valCK},
    vp=setPrecLame[v];
    kp=setPrecLame[k];
    valCK=Sqrt[1-k]*Sum[(2 h+1)*LameCoefficientB[2 m+1,vp,kp,2 h+1],{h,0,(v-1)/2}];
  If[Re[valCK]>0,1,-1]];

(*------------------- Lame polynomial function cdE --------------------*)

LamePolynomialCDE[v_,m_Integer,{z_,k_}]:=
  Module[{vp,zp,kp,phi},
    vp=setPrecLame[v];
    zp=setPrecLame[z];
    kp=setPrecLame[k];
    phi=setPrecLame[Pi/2-am[zp,kp]];
    N[Sum[LameCoefficientD[2 m+1,vp,kp,2 h+1]*Sin[(2 h+1)*phi],{h,0,v/2-1}]]]*
      JacobiDN[z,k]*sgnFactorPolyCD[v,m,k]/;
  (EvenQ[Round[v]] && 0<=m<=(v/2-1) && Precision[z]<Infinity)

(* sgnFactorPolyCD[v,n,k] *)
sgnFactorPolyCD[v_,m_Integer,k_]:=
  Module[{vp,kp,valCDK},
    vp=setPrecLame[v];
    kp=setPrecLame[k];
    valCDK=(1-k)*Sum[(2 h+1)*LameCoefficientD[2 m+1,vp,kp,2 h+1],{h,0,v/2-1}];
    If[Re[valCDK]>0,1,-1]];

(*-------------- Derivative Lame polynomial function uE ---------------*)

LamePolynomialUEPrime[v_,m_Integer,{z_,k_}]:=
  Module[{vp,zp,kp,phi},
    vp=setPrecLame[v];
    zp=setPrecLame[z];
    kp=setPrecLame[k];
    phi=setPrecLame[Pi/2-am[zp,kp]];
    N[Sum[2 h*LameCoefficientA[2 m,vp,kp,2 h]*Sin[(2 h)*phi],{h,1,v/2}]]]*
      JacobiDN[z,k]*sgnFactorPolyU[v,m,k]/;
  (EvenQ[Round[v]] && 0<=m<=v/2 && Precision[z]<Infinity)

(*-------------- Derivative Lame polynomial function dE ---------------*)

LamePolynomialDEPrime[v_,m_Integer,{z_,k_}]:=
  Module[{vp,zp,kp,phi},
    vp=setPrecLame[v];
    zp=setPrecLame[z];
    kp=setPrecLame[k];
    phi=setPrecLame[Pi/2-am[zp,kp]];
    N[-kp*JacobiSN[zp,kp]*JacobiCN[zp,kp]*
      (Sum[LameCoefficientC[2 m,vp,kp,2 h]*Cos[(2 h)*phi],{h,1,(v-1)/2}]+
      LameCoefficientC[2 m,vp,kp,0]/2)+JacobiDN[zp,kp]^2*
      Sum[2 h*LameCoefficientC[2 m,vp,kp,2 h]*Sin[(2 h)*phi],{h,1,(v-1)/2}]]]*
      sgnFactorPolyD[v,m,k]/;
  (OddQ[Round[v]] && 0<=m<=(v-1)/2 && Precision[z]<Infinity)

(*-------------- Derivative Lame polynomial function sdE --------------*)

LamePolynomialSDEPrime[v_,m_Integer,{z_,k_}]:=
  Module[{vp,zp,kp,phi},
    vp=setPrecLame[v];
    zp=setPrecLame[z];
    kp=setPrecLame[k];
    phi=setPrecLame[Pi/2-am[zp,kp]];
    N[-kp*JacobiSN[zp,kp]*JacobiCN[zp,kp]*
      Sum[LameCoefficientC[2 m+1,vp,kp,2 h+1]*Cos[(2 h+1)*phi],{h,0,v/2-1}]+
      JacobiDN[zp,kp]^2*
      Sum[(2 h+1)*LameCoefficientC[2 m+1,vp,kp,2 h+1]*Sin[(2 h+1)*phi],{h,0,v/2-1}]]]*
      sgnFactorPolySD[v,m,k]/;
  (EvenQ[Round[v]] && 0<=m<=(v/2-1) && Precision[z]<Infinity)

(*-------------- Derivative Lame polynomial function sE ---------------*)

LamePolynomialSEPrime[v_,m_Integer,{z_,k_}]:=
  Module[{vp,zp,kp,phi},
    vp=setPrecLame[v];
    zp=setPrecLame[z];
    kp=setPrecLame[k];
    phi=setPrecLame[Pi/2-am[zp,kp]];
    N[Sum[(2 h+1)*LameCoefficientA[2 m+1,vp,kp,2 h+1]*Sin[(2 h+1)*phi],{h,0,(v-1)/2}]]]*
      JacobiDN[z,k]*sgnFactorPolyS[v,m,k]/;
  (OddQ[Round[v]] && 0<=m<=(v-1)/2 && Precision[z]<Infinity)

(*-------------- Derivative Lame polynomial function scE --------------*)

LamePolynomialSCEPrime[v_,m_Integer,{z_,k_}]:=
  Module[{vp,zp,kp,phi},
    vp=setPrecLame[v];
    zp=setPrecLame[z];
    kp=setPrecLame[k];
    phi=setPrecLame[Pi/2-am[zp,kp]];
    N[Sum[-(2 h+2)*LameCoefficientB[2 m+2,vp,kp,2 h+2]*Cos[(2 h+2)*phi],{h,0,v/2-1}]]]*
      JacobiDN[z,k]*sgnFactorPolySC[v,m,k]/;
  (EvenQ[Round[v]] && 0<=m<=(v/2-1) && Precision[z]<Infinity)

(*------------- Derivative Lame polynomial function scdE --------------*)

LamePolynomialSCDEPrime[v_,m_Integer,{z_,k_}]:=
  Module[{vp,zp,kp,phi},
    vp=setPrecLame[v];
    zp=setPrecLame[z];
    kp=setPrecLame[k];
    phi=setPrecLame[Pi/2-am[zp,kp]];
    N[-kp*JacobiSN[zp,kp]*JacobiCN[zp,kp]*
      Sum[LameCoefficientD[2 m+2,vp,kp,2 h+2]*Sin[(2 h+2)*phi],{h,0,(v-3)/2}]-
      JacobiDN[zp,kp]^2*
      Sum[(2 h+2)*LameCoefficientD[2 m+2,vp,kp,2 h+2]*Cos[(2 h+2)*phi],{h,0,(v-3)/2}]]]*
      sgnFactorPolySCD[v,m,k]/;
  (OddQ[Round[v]] && 0<=m<=(v-3)/2 && Precision[z]<Infinity)

(*-------------- Derivative Lame polynomial function cE ---------------*)

LamePolynomialCEPrime[v_,m_Integer,{z_,k_}]:=
  Module[{vp,zp,kp,phi},
    vp=setPrecLame[v];
    zp=setPrecLame[z];
    kp=setPrecLame[k];
    phi=setPrecLame[Pi/2-am[zp,kp]];
    N[Sum[-(2 h+1)*LameCoefficientB[2 m+1,vp,kp,2 h+1]*Cos[(2 h+1)*phi],{h,0,(v-1)/2}]]]*
      JacobiDN[z,k]*sgnFactorPolyC[v,m,k]/;
  (OddQ[Round[v]] && 0<=m<=(v-1)/2 && Precision[z]<Infinity)

(*-------------- Derivative Lame polynomial function cdE --------------*)

LamePolynomialCDEPrime[v_,m_Integer,{z_,k_}]:=
  Module[{vp,zp,kp,phi},
    vp=setPrecLame[v];
    zp=setPrecLame[z];
    kp=setPrecLame[k];
    phi=setPrecLame[Pi/2-am[zp,kp]];
    N[-kp*JacobiSN[zp,kp]*JacobiCN[zp,kp]*
      Sum[LameCoefficientD[2 m+1,vp,kp,2 h+1]*Sin[(2 h+1)*phi],{h,0,v/2-1}]-
      JacobiDN[zp,kp]^2*
      Sum[(2 h+1)*LameCoefficientD[2 m+1,vp,kp,2 h+1]*Cos[(2 h+1)*phi],{h,0,v/2-1}]]]*
      sgnFactorPolyCD[v,m,k]/;
  (EvenQ[Round[v]] && 0<=m<=(v/2-1) && Precision[z]<Infinity)



(**********************************************************************************)
(*                           Generalized Lame functions                           *)
(**********************************************************************************)

(*--------------------- Speed up : Initial values ---------------------*)
initLamK[h_,v_,k_,{w0_,w1_}]:=
  initLamK[h,v,k,{w0,w1}]=
    Lame[h,v,{N[EllipticK[k]],k},{w0,w1}];

initLamPrimeK[h_,v_,k_,{w0_,w1_}]:=
  initLamPrimeK[h,v,k,{w0,w1}]=
    LamePrime[h,v,{N[EllipticK[k]],k},{w0,w1}];

initGK[h_,v_,k_,{w0_,w1_}]:=
  initGK[h,v,k,{w0,w1}]=
    GeneralizedLame[h,v,{N[EllipticK[k]],k},{w0,w1}];

initGKM[h_,v_,k_,{w0_,w1_}]:=
  initGKM[h,v,k,{w0,w1}]=
    GeneralizedLame[h,v,{-N[EllipticK[k]],k},{w0,w1}];

initGPrimeK[h_,v_,k_,{w0_,w1_}]:=
  initGPrimeK[h,v,k,{w0,w1}]=
    GeneralizedLamePrime[h,v,{N[EllipticK[k]],k},{w0,w1}];

initGPrimeKM[h_,v_,k_,{w0_,w1_}]:=
  initGPrimeKM[h,v,k,{w0,w1}]=
    GeneralizedLamePrime[h,v,{-N[EllipticK[k]],k},{w0,w1}];

initAK[h_,v_,k_,{w0_,w1_}]:=
  initAK[h,v,k,{w0,w1}]=
    AuxiliaryLame[h,v,{N[EllipticK[k]],k},{w0,w1}];

initAKM[h_,v_,k_,{w0_,w1_}]:=
  initAKM[h,v,k,{w0,w1}]=
    AuxiliaryLame[h,v,{-N[EllipticK[k]],k},{w0,w1}];

initAPrimeK[h_,v_,k_,{w0_,w1_}]:=
  initAPrimeK[h,v,k,{w0,w1}]=
    AuxiliaryLamePrime[h,v,{N[EllipticK[k]],k},{w0,w1}];

initAPrimeKM[h_,v_,k_,{w0_,w1_}]:=
  initAPrimeKM[h,v,k,{w0,w1}]=
    AuxiliaryLamePrime[h,v,{-N[EllipticK[k]],k},{w0,w1}];

(*-------------- Generalized Lame function L[h,v][z,k][w0,w1] --------------*)
(* Complex variables "z" *)
GeneralizedLame[h_,v_,{z_,k_},{w0_,w1_}]:=
  Module[{f0K,f1K},
    f0K=initLamK[h,v,k,{w0,w1}];
    f1K=initLamPrimeK[h,v,k,{w0,w1}];
    LameK[h,v,{z,k},{f0K,f1K}]]/;
  (Abs[Im[z]]>0 && 0<=Re[z]<=N[2*EllipticK[k]] && Precision[z]<Infinity)

GeneralizedLame[h_,v_,{z_,k_},{w0_,w1_}]:=
  Module[{eK,gK,gKm,gpK,gpKm,aKm,apKm,cons1,cons2},
    eK=N[EllipticK[k]];
    gK=initGK[h,v,k,{w0,w1}];
    gKm=initGKM[h,v,k,{w0,w1}];
    gpK=initGPrimeK[h,v,k,{w0,w1}];
    gpKm=initGPrimeKM[h,v,k,{w0,w1}];
    aKm=initAKM[h,v,k,{w0,w1}];
    apKm=initAPrimeKM[h,v,k,{w0,w1}];
    cons1=(gK*apKm-gpK*aKm)/(gKm*apKm-gpKm*aKm);
    cons2=(gK*gpKm-gpK*gKm)/(aKm*gpKm-apKm*gKm);
    cons1*GeneralizedLame[h,v,{z-2 eK,k},{w0,w1}]+
      cons2*AuxiliaryLame[h,v,{z-2 eK,k},{w0,w1}]]/;
  (Abs[Im[z]]>0 && Re[z]>N[2*EllipticK[k]] && Precision[z]<Infinity)

GeneralizedLame[h_,v_,{z_,k_},{w0_,w1_}]:=
  Module[{eK,gK,gKm,gpK,gpKm,aK,apK,cons1,cons2},
    eK=N[EllipticK[k]];
    gK=initGK[h,v,k,{w0,w1}];
    gKm=initGKM[h,v,k,{w0,w1}];
    gpK=initGPrimeK[h,v,k,{w0,w1}];
    gpKm=initGPrimeKM[h,v,k,{w0,w1}];
    aK=initAK[h,v,k,{w0,w1}];
    apK=initAPrimeK[h,v,k,{w0,w1}];
    cons1=(gKm*apK-gpKm*aK)/(gK*apK-gpK*aK);
    cons2=(gKm*gpK-gpKm*gK)/(aK*gpK-apK*gK);
    cons1*GeneralizedLame[h,v,{z+2 eK,k},{w0,w1}]+
      cons2*AuxiliaryLame[h,v,{z+2 eK,k},{w0,w1}]]/;
  (Abs[Im[z]]>0 && Re[z]<0 && Precision[z]<Infinity)

(* Real variables "z" *)
GeneralizedLame[h_,v_,{z_,k_},{w0_,w1_}]:=
  Lame[h,v,{z,k},{w0,w1}]/;
  (N[Im[z]]===0. && Abs[Re[z]]<=N[EllipticK[k]] && Precision[z]<Infinity)

GeneralizedLame[h_,v_,{z_,k_},{w0_,w1_}]:=
  Module[{eK,gK,gKm,gpK,gpKm,aKm,apKm,cons1,cons2},
    eK=N[EllipticK[k]];
    gK=initGK[h,v,k,{w0,w1}];
    gKm=initGKM[h,v,k,{w0,w1}];
    gpK=initGPrimeK[h,v,k,{w0,w1}];
    gpKm=initGPrimeKM[h,v,k,{w0,w1}];
    aKm=initAKM[h,v,k,{w0,w1}];
    apKm=initAPrimeKM[h,v,k,{w0,w1}];
    cons1=(gK*apKm-gpK*aKm)/(gKm*apKm-gpKm*aKm);
    cons2=(gK*gpKm-gpK*gKm)/(aKm*gpKm-apKm*gKm);
    cons1*GeneralizedLame[h,v,{z-2 eK,k},{w0,w1}]+
      cons2*AuxiliaryLame[h,v,{z-2 eK,k},{w0,w1}]]/;
  (N[Im[z]]===0. && Re[z]>N[EllipticK[k]] && Precision[z]<Infinity)

GeneralizedLame[h_,v_,{z_,k_},{w0_,w1_}]:=
  Module[{eK,gK,gKm,gpK,gpKm,aK,apK,cons1,cons2},
    eK=N[EllipticK[k]];
    gK=initGK[h,v,k,{w0,w1}];
    gKm=initGKM[h,v,k,{w0,w1}];
    gpK=initGPrimeK[h,v,k,{w0,w1}];
    gpKm=initGPrimeKM[h,v,k,{w0,w1}];
    aK=initAK[h,v,k,{w0,w1}];
    apK=initAPrimeK[h,v,k,{w0,w1}];
    cons1=(gKm*apK-gpKm*aK)/(gK*apK-gpK*aK);
    cons2=(gKm*gpK-gpKm*gK)/(aK*gpK-apK*gK);
    cons1*GeneralizedLame[h,v,{z+2 eK,k},{w0,w1}]+
      cons2*AuxiliaryLame[h,v,{z+2 eK,k},{w0,w1}]]/;
  (N[Im[z]]===0. && Re[z]<-N[EllipticK[k]] && Precision[z]<Infinity)

(*-------------- Auxiliary Lame function L2[h,v][z,k][w0,w1] ---------------*)
(* Complex variables "z" *)
AuxiliaryLame[h_,v_,{z_,k_},{w0_,w1_}]:=
  Module[{f0K,f1K},
    f0K=initLamK[h,v,k,{w0-1,w1+1}];
    f1K=initLamPrimeK[h,v,k,{w0-1,w1+1}];
    LameK[h,v,{z,k},{f0K,f1K}]]/;
  (Abs[Im[z]]>0 && 0<=Re[z]<=N[2*EllipticK[k]] && Precision[z]<Infinity)

AuxiliaryLame[h_,v_,{z_,k_},{w0_,w1_}]:=
  Module[{eK,aK,aKm,apK,apKm,gKm,gpKm,cons1,cons2},
    eK=N[EllipticK[k]];
    aK=initAK[h,v,k,{w0,w1}];
    aKm=initAKM[h,v,k,{w0,w1}];
    apK=initAPrimeK[h,v,k,{w0,w1}];
    apKm=initAPrimeKM[h,v,k,{w0,w1}];
    gKm=initGKM[h,v,k,{w0,w1}];
    gpKm=initGPrimeKM[h,v,k,{w0,w1}];
    cons1=(aK*gpKm-apK*gKm)/(aKm*gpKm-apKm*gKm);
    cons2=(aK*apKm-apK*aKm)/(gKm*apKm-gpKm*aKm);
    cons1*AuxiliaryLame[h,v,{z-2 eK,k},{w0,w1}]+
      cons2*GeneralizedLame[h,v,{z-2 eK,k},{w0,w1}]]/;
  (Abs[Im[z]]>0 && Re[z]>N[2*EllipticK[k]] && Precision[z]<Infinity)

AuxiliaryLame[h_,v_,{z_,k_},{w0_,w1_}]:=
  Module[{eK,aK,aKm,apK,apKm,gK,gpK,cons1,cons2},
    eK=N[EllipticK[k]];
    aK=initAK[h,v,k,{w0,w1}];
    aKm=initAKM[h,v,k,{w0,w1}];
    apK=initAPrimeK[h,v,k,{w0,w1}];
    apKm=initAPrimeKM[h,v,k,{w0,w1}];
    gK=initGK[h,v,k,{w0,w1}];
    gpK=initGPrimeK[h,v,k,{w0,w1}];
    cons1=(aKm*gpK-apKm*gK)/(aK*gpK-apK*gK);
    cons2=(aKm*apK-apKm*aK)/(gK*apK-gpK*aK);
    cons1*AuxiliaryLame[h,v,{z+2 eK,k},{w0,w1}]+
      cons2*GeneralizedLame[h,v,{z+2 eK,k},{w0,w1}]]/;
  (Abs[Im[z]]>0 && Re[z]<0 && Precision[z]<Infinity)

(* Real variables "z" *)
AuxiliaryLame[h_,v_,{z_,k_},{w0_,w1_}]:=
  Lame[h,v,{z,k},{w0-1,w1+1}]/;
  (N[Im[z]]===0. && Abs[Re[z]]<=N[EllipticK[k]] && Precision[z]<Infinity)

AuxiliaryLame[h_,v_,{z_,k_},{w0_,w1_}]:=
  Module[{eK,aK,aKm,apK,apKm,gKm,gpKm,cons1,cons2},
    eK=N[EllipticK[k]];
    aK=initAK[h,v,k,{w0,w1}];
    aKm=initAKM[h,v,k,{w0,w1}];
    apK=initAPrimeK[h,v,k,{w0,w1}];
    apKm=initAPrimeKM[h,v,k,{w0,w1}];
    gKm=initGKM[h,v,k,{w0,w1}];
    gpKm=initGPrimeKM[h,v,k,{w0,w1}];
    cons1=(aK*gpKm-apK*gKm)/(aKm*gpKm-apKm*gKm);
    cons2=(aK*apKm-apK*aKm)/(gKm*apKm-gpKm*aKm);
    cons1*AuxiliaryLame[h,v,{z-2 eK,k},{w0,w1}]+
      cons2*GeneralizedLame[h,v,{z-2 eK,k},{w0,w1}]]/;
  (N[Im[z]]===0. && Re[z]>N[EllipticK[k]] && Precision[z]<Infinity)

AuxiliaryLame[h_,v_,{z_,k_},{w0_,w1_}]:=
  Module[{eK,aK,aKm,apK,apKm,gK,gpK,cons1,cons2},
    eK=N[EllipticK[k]];
    aK=initAK[h,v,k,{w0,w1}];
    aKm=initAKM[h,v,k,{w0,w1}];
    apK=initAPrimeK[h,v,k,{w0,w1}];
    apKm=initAPrimeKM[h,v,k,{w0,w1}];
    gK=initGK[h,v,k,{w0,w1}];
    gpK=initGPrimeK[h,v,k,{w0,w1}];
    cons1=(aKm*gpK-apKm*gK)/(aK*gpK-apK*gK);
    cons2=(aKm*apK-apKm*aK)/(gK*apK-gpK*aK);
    cons1*AuxiliaryLame[h,v,{z+2 eK,k},{w0,w1}]+
      cons2*GeneralizedLame[h,v,{z+2 eK,k},{w0,w1}]]/;
  (N[Im[z]]===0. && Re[z]<-N[EllipticK[k]] && Precision[z]<Infinity)

(*-------- Derivative Generalized Lame function L'[h,v][z,k][w0,w1] --------*)
(* Complex variables "z" *)
GeneralizedLamePrime[h_,v_,{z_,k_},{w0_,w1_}]:=
  Module[{f0K,f1K},
    f0K=initLamK[h,v,k,{w0,w1}];
    f1K=initLamPrimeK[h,v,k,{w0,w1}];
    LamePrimeK[h,v,{z,k},{f0K,f1K}]]/;
  (Abs[Im[z]]>0 && 0<=Re[z]<=N[2*EllipticK[k]] && Precision[z]<Infinity)

GeneralizedLamePrime[h_,v_,{z_,k_},{w0_,w1_}]:=
  Module[{eK,gK,gKm,gpK,gpKm,aKm,apKm,cons1,cons2},
    eK=N[EllipticK[k]];
    gK=initGK[h,v,k,{w0,w1}];
    gKm=initGKM[h,v,k,{w0,w1}];
    gpK=initGPrimeK[h,v,k,{w0,w1}];
    gpKm=initGPrimeKM[h,v,k,{w0,w1}];
    aKm=initAKM[h,v,k,{w0,w1}];
    apKm=initAPrimeKM[h,v,k,{w0,w1}];
    cons1=(gK*apKm-gpK*aKm)/(gKm*apKm-gpKm*aKm);
    cons2=(gK*gpKm-gpK*gKm)/(aKm*gpKm-apKm*gKm);
    cons1*GeneralizedLamePrime[h,v,{z-2 eK,k},{w0,w1}]+
      cons2*AuxiliaryLamePrime[h,v,{z-2 eK,k},{w0,w1}]]/;
  (Abs[Im[z]]>0 && Re[z]>N[2*EllipticK[k]] && Precision[z]<Infinity)

GeneralizedLamePrime[h_,v_,{z_,k_},{w0_,w1_}]:=
  Module[{eK,gK,gKm,gpK,gpKm,aK,apK,cons1,cons2},
    eK=N[EllipticK[k]];
    gK=initGK[h,v,k,{w0,w1}];
    gKm=initGKM[h,v,k,{w0,w1}];
    gpK=initGPrimeK[h,v,k,{w0,w1}];
    gpKm=initGPrimeKM[h,v,k,{w0,w1}];
    aK=initAK[h,v,k,{w0,w1}];
    apK=initAPrimeK[h,v,k,{w0,w1}];
    cons1=(gKm*apK-gpKm*aK)/(gK*apK-gpK*aK);
    cons2=(gKm*gpK-gpKm*gK)/(aK*gpK-apK*gK);
    cons1*GeneralizedLamePrime[h,v,{z+2 eK,k},{w0,w1}]+
      cons2*AuxiliaryLamePrime[h,v,{z+2 eK,k},{w0,w1}]]/;
  (Abs[Im[z]]>0 && Re[z]<0 && Precision[z]<Infinity)

(* Real variables "z" *)
GeneralizedLamePrime[h_,v_,{z_,k_},{w0_,w1_}]:=
  LamePrime[h,v,{z,k},{w0,w1}]/;
  (N[Im[z]]===0. && Abs[Re[z]]<=N[EllipticK[k]] && Precision[z]<Infinity)

GeneralizedLamePrime[h_,v_,{z_,k_},{w0_,w1_}]:=
  Module[{eK,gK,gKm,gpK,gpKm,aKm,apKm,cons1,cons2},
    eK=N[EllipticK[k]];
    gK=initGK[h,v,k,{w0,w1}];
    gKm=initGKM[h,v,k,{w0,w1}];
    gpK=initGPrimeK[h,v,k,{w0,w1}];
    gpKm=initGPrimeKM[h,v,k,{w0,w1}];
    aKm=initAKM[h,v,k,{w0,w1}];
    apKm=initAPrimeKM[h,v,k,{w0,w1}];
    cons1=(gK*apKm-gpK*aKm)/(gKm*apKm-gpKm*aKm);
    cons2=(gK*gpKm-gpK*gKm)/(aKm*gpKm-apKm*gKm);
    cons1*GeneralizedLamePrime[h,v,{z-2 eK,k},{w0,w1}]+
      cons2*AuxiliaryLamePrime[h,v,{z-2 eK,k},{w0,w1}]]/;
  (N[Im[z]]===0. && Re[z]>N[EllipticK[k]] && Precision[z]<Infinity)

GeneralizedLamePrime[h_,v_,{z_,k_},{w0_,w1_}]:=
  Module[{eK,gK,gKm,gpK,gpKm,aK,apK,cons1,cons2},
    eK=N[EllipticK[k]];
    gK=initGK[h,v,k,{w0,w1}];
    gKm=initGKM[h,v,k,{w0,w1}];
    gpK=initGPrimeK[h,v,k,{w0,w1}];
    gpKm=initGPrimeKM[h,v,k,{w0,w1}];
    aK=initAK[h,v,k,{w0,w1}];
    apK=initAPrimeK[h,v,k,{w0,w1}];
    cons1=(gKm*apK-gpKm*aK)/(gK*apK-gpK*aK);
    cons2=(gKm*gpK-gpKm*gK)/(aK*gpK-apK*gK);
    cons1*GeneralizedLamePrime[h,v,{z+2 eK,k},{w0,w1}]+
      cons2*AuxiliaryLamePrime[h,v,{z+2 eK,k},{w0,w1}]]/;
  (N[Im[z]]===0. && Re[z]<-N[EllipticK[k]] && Precision[z]<Infinity)

(*-------- Derivative Auxiliary Lame function L2'[h,v][z,k][w0,w1] ---------*)
(* Complex variables "z" *)
AuxiliaryLamePrime[h_,v_,{z_,k_},{w0_,w1_}]:=
  Module[{f0K,f1K},
    f0K=initLamK[h,v,k,{w0-1,w1+1}];
    f1K=initLamPrimeK[h,v,k,{w0-1,w1+1}];
    LamePrimeK[h,v,{z,k},{f0K,f1K}]]/;
  (Abs[Im[z]]>0 && 0<=Re[z]<=N[2*EllipticK[k]] && Precision[z]<Infinity)

AuxiliaryLamePrime[h_,v_,{z_,k_},{w0_,w1_}]:=
  Module[{eK,aK,aKm,apK,apKm,gKm,gpKm,cons1,cons2},
    eK=N[EllipticK[k]];
    aK=initAK[h,v,k,{w0,w1}];
    aKm=initAKM[h,v,k,{w0,w1}];
    apK=initAPrimeK[h,v,k,{w0,w1}];
    apKm=initAPrimeKM[h,v,k,{w0,w1}];
    gKm=initGKM[h,v,k,{w0,w1}];
    gpKm=initGPrimeKM[h,v,k,{w0,w1}];
    cons1=(aK*gpKm-apK*gKm)/(aKm*gpKm-apKm*gKm);
    cons2=(aK*apKm-apK*aKm)/(gKm*apKm-gpKm*aKm);
    cons1*AuxiliaryLamePrime[h,v,{z-2 eK,k},{w0,w1}]+
      cons2*GeneralizedLamePrime[h,v,{z-2 eK,k},{w0,w1}]]/;
  (Abs[Im[z]]>0 && Re[z]>N[2*EllipticK[k]] && Precision[z]<Infinity)

AuxiliaryLamePrime[h_,v_,{z_,k_},{w0_,w1_}]:=
  Module[{eK,aK,aKm,apK,apKm,gK,gpK,cons1,cons2},
    eK=N[EllipticK[k]];
    aK=initAK[h,v,k,{w0,w1}];
    aKm=initAKM[h,v,k,{w0,w1}];
    apK=initAPrimeK[h,v,k,{w0,w1}];
    apKm=initAPrimeKM[h,v,k,{w0,w1}];
    gK=initGK[h,v,k,{w0,w1}];
    gpK=initGPrimeK[h,v,k,{w0,w1}];
    cons1=(aKm*gpK-apKm*gK)/(aK*gpK-apK*gK);
    cons2=(aKm*apK-apKm*aK)/(gK*apK-gpK*aK);
    cons1*AuxiliaryLamePrime[h,v,{z+2 eK,k},{w0,w1}]+
      cons2*GeneralizedLamePrime[h,v,{z+2 eK,k},{w0,w1}]]/;
  (Abs[Im[z]]>0 && Re[z]<0 && Precision[z]<Infinity)

(* Real variables "z" *)
AuxiliaryLamePrime[h_,v_,{z_,k_},{w0_,w1_}]:=
  LamePrime[h,v,{z,k},{w0-1,w1+1}]/;
  (N[Im[z]]===0. && Abs[Re[z]]<=N[EllipticK[k]] && Precision[z]<Infinity)

AuxiliaryLamePrime[h_,v_,{z_,k_},{w0_,w1_}]:=
  Module[{eK,aK,aKm,apK,apKm,gKm,gpKm,cons1,cons2},
    eK=N[EllipticK[k]];
    aK=initAK[h,v,k,{w0,w1}];
    aKm=initAKM[h,v,k,{w0,w1}];
    apK=initAPrimeK[h,v,k,{w0,w1}];
    apKm=initAPrimeKM[h,v,k,{w0,w1}];
    gKm=initGKM[h,v,k,{w0,w1}];
    gpKm=initGPrimeKM[h,v,k,{w0,w1}];
    cons1=(aK*gpKm-apK*gKm)/(aKm*gpKm-apKm*gKm);
    cons2=(aK*apKm-apK*aKm)/(gKm*apKm-gpKm*aKm);
    cons1*AuxiliaryLamePrime[h,v,{z-2 eK,k},{w0,w1}]+
      cons2*GeneralizedLamePrime[h,v,{z-2 eK,k},{w0,w1}]]/;
  (N[Im[z]]===0. && Re[z]>N[EllipticK[k]] && Precision[z]<Infinity)

AuxiliaryLamePrime[h_,v_,{z_,k_},{w0_,w1_}]:=
  Module[{eK,aK,aKm,apK,apKm,gK,gpK,cons1,cons2},
    eK=N[EllipticK[k]];
    aK=initAK[h,v,k,{w0,w1}];
    aKm=initAKM[h,v,k,{w0,w1}];
    apK=initAPrimeK[h,v,k,{w0,w1}];
    apKm=initAPrimeKM[h,v,k,{w0,w1}];
    gK=initGK[h,v,k,{w0,w1}];
    gpK=initGPrimeK[h,v,k,{w0,w1}];
    cons1=(aKm*gpK-apKm*gK)/(aK*gpK-apK*gK);
    cons2=(aKm*apK-apKm*aK)/(gK*apK-gpK*aK);
    cons1*AuxiliaryLamePrime[h,v,{z+2 eK,k},{w0,w1}]+
      cons2*GeneralizedLamePrime[h,v,{z+2 eK,k},{w0,w1}]]/;
  (N[Im[z]]===0. && Re[z]<-N[EllipticK[k]] && Precision[z]<Infinity)



(**********************************************************************************)
(*                        Generalized Lame wave functions                         *)
(**********************************************************************************)

(*------- "NDSolve" : Methods for values of Lame wave function --------*)
LameWave[h_,q_,v_,{z_,k_},{w0_,w1_}]:=
  Module[{sol,f,x},
    sol=NDSolve[{f''[x]+(h-v*(v+1)*k*sn[x,k]^2+q*k^2*sn[x,k]^4)*f[x]==0,
      f[0]==w0,f'[0]==w1},f,{x,z},MaxStepSize->0.01];
  f[Re[z]]/.sol[[1]]];

LameWavePrime[h_,q_,v_,{z_,k_},{w0_,w1_}]:=
  Module[{sol,f,x},
    sol=NDSolve[{f''[x]+(h-v*(v+1)*k*sn[x,k]^2+q*k^2*sn[x,k]^4)*f[x]==0,
      f[0]==w0,f'[0]==w1},f,{x,z},MaxStepSize->0.01];
  f'[Re[z]]/.sol[[1]]];

(*------ "NDSolve" of Lame wave function : Initial point at z=K -------*)
LameWaveK[h_,q_,v_,{z_,k_},{w0_,w1_}]:=
  Module[{theta,sol,f,x,shift=N[EllipticK[k]]},
    theta=If[z==shift,0,Arg[z-shift]];
    sol=NDSolve[{Exp[-2 I*theta]*f''[x]+
    (h-v*(v+1)*k*sn[Exp[I*theta]*x+shift,k]^2+q*k^2*sn[Exp[I*theta]*x+shift,k]^4)*f[x]==0,
    f[0]==w0,f'[0]==Exp[I*theta]*w1},f,{x,Abs[z-shift]}];
  f[Abs[z-shift]]/.sol[[1]]]/;
  (Precision[z]<Infinity)

LameWavePrimeK[h_,q_,v_,{z_,k_},{w0_,w1_}]:=
  Module[{theta,sol,f,x,shift=N[EllipticK[k]]},
    theta=If[z==shift,0,Arg[z-shift]];
    sol=NDSolve[{Exp[-2 I*theta]*f''[x]+
    (h-v*(v+1)*k*sn[Exp[I*theta]*x+shift,k]^2+q*k^2*sn[Exp[I*theta]*x+shift,k]^4)*f[x]==0,
    f[0]==w0,f'[0]==Exp[I*theta]*w1},f,{x,Abs[z-shift]}];
  Exp[-I*theta]*f'[Abs[z-shift]]/.sol[[1]]]/;
  (Precision[z]<Infinity)

(*--------------------- Speed up : Initial values ---------------------*)
initLamWaveK[h_,q_,v_,k_,{w0_,w1_}]:=
  initLamWaveK[h,q,v,k,{w0,w1}]=
    LameWave[h,q,v,{N[EllipticK[k]],k},{w0,w1}];

initLamWavePrimeK[h_,q_,v_,k_,{w0_,w1_}]:=
  initLamWavePrimeK[h,q,v,k,{w0,w1}]=
    LameWavePrime[h,q,v,{N[EllipticK[k]],k},{w0,w1}];

initGWaveK[h_,q_,v_,k_,{w0_,w1_}]:=
  initGWaveK[h,q,v,k,{w0,w1}]=
    GeneralizedLameWave[h,q,v,{N[EllipticK[k]],k},{w0,w1}];

initGWaveKM[h_,q_,v_,k_,{w0_,w1_}]:=
  initGWaveKM[h,q,v,k,{w0,w1}]=
    GeneralizedLameWave[h,q,v,{-N[EllipticK[k]],k},{w0,w1}];

initGWavePrimeK[h_,q_,v_,k_,{w0_,w1_}]:=
  initGWavePrimeK[h,q,v,k,{w0,w1}]=
    GeneralizedLameWavePrime[h,q,v,{N[EllipticK[k]],k},{w0,w1}];

initGWavePrimeKM[h_,q_,v_,k_,{w0_,w1_}]:=
  initGWavePrimeKM[h,q,v,k,{w0,w1}]=
    GeneralizedLameWavePrime[h,q,v,{-N[EllipticK[k]],k},{w0,w1}];

initAWaveK[h_,q_,v_,k_,{w0_,w1_}]:=
  initAWaveK[h,q,v,k,{w0,w1}]=
    AuxiliaryLameWave[h,q,v,{N[EllipticK[k]],k},{w0,w1}];

initAWaveKM[h_,q_,v_,k_,{w0_,w1_}]:=
  initAWaveKM[h,q,v,k,{w0,w1}]=
    AuxiliaryLameWave[h,q,v,{-N[EllipticK[k]],k},{w0,w1}];

initAWavePrimeK[h_,q_,v_,k_,{w0_,w1_}]:=
  initAWavePrimeK[h,q,v,k,{w0,w1}]=
    AuxiliaryLameWavePrime[h,q,v,{N[EllipticK[k]],k},{w0,w1}];

initAWavePrimeKM[h_,q_,v_,k_,{w0_,w1_}]:=
  initAWavePrimeKM[h,q,v,k,{w0,w1}]=
    AuxiliaryLameWavePrime[h,q,v,{-N[EllipticK[k]],k},{w0,w1}];

(*--------- Generalized Lame wave function E[h,v][z,k][w0,w1] ---------*)
(* Complex variables "z" *)
GeneralizedLameWave[h_,q_,v_,{z_,k_},{w0_,w1_}]:=
  Module[{f0K,f1K},
    f0K=initLamWaveK[h,q,v,k,{w0,w1}];
    f1K=initLamWavePrimeK[h,q,v,k,{w0,w1}];
    LameWaveK[h,q,v,{z,k},{f0K,f1K}]]/;
  (Abs[Im[z]]>0 && 0<=Re[z]<=N[2*EllipticK[k]] && Precision[z]<Infinity)

GeneralizedLameWave[h_,q_,v_,{z_,k_},{w0_,w1_}]:=
  Module[{eK,gK,gKm,gpK,gpKm,aKm,apKm,cons1,cons2},
    eK=N[EllipticK[k]];
    gK=initGWaveK[h,q,v,k,{w0,w1}];
    gKm=initGWaveKM[h,q,v,k,{w0,w1}];
    gpK=initGWavePrimeK[h,q,v,k,{w0,w1}];
    gpKm=initGWavePrimeKM[h,q,v,k,{w0,w1}];
    aKm=initAWaveKM[h,q,v,k,{w0,w1}];
    apKm=initAWavePrimeKM[h,q,v,k,{w0,w1}];
    cons1=(gK*apKm-gpK*aKm)/(gKm*apKm-gpKm*aKm);
    cons2=(gK*gpKm-gpK*gKm)/(aKm*gpKm-apKm*gKm);
    cons1*GeneralizedLameWave[h,q,v,{z-2 eK,k},{w0,w1}]+
    cons2*AuxiliaryLameWave[h,q,v,{z-2 eK,k},{w0,w1}]]/;
  (Abs[Im[z]]>0 && Re[z]>N[2*EllipticK[k]] && Precision[z]<Infinity)

GeneralizedLameWave[h_,q_,v_,{z_,k_},{w0_,w1_}]:=
  Module[{eK,gK,gKm,gpK,gpKm,aK,apK,cons1,cons2},
    eK=N[EllipticK[k]];
    gK=initGWaveK[h,q,v,k,{w0,w1}];
    gKm=initGWaveKM[h,q,v,k,{w0,w1}];
    gpK=initGWavePrimeK[h,q,v,k,{w0,w1}];
    gpKm=initGWavePrimeKM[h,q,v,k,{w0,w1}];
    aK=initAWaveK[h,q,v,k,{w0,w1}];
    apK=initAWavePrimeK[h,q,v,k,{w0,w1}];
    cons1=(gKm*apK-gpKm*aK)/(gK*apK-gpK*aK);
    cons2=(gKm*gpK-gpKm*gK)/(aK*gpK-apK*gK);
    cons1*GeneralizedLameWave[h,q,v,{z+2 eK,k},{w0,w1}]+
    cons2*AuxiliaryLameWave[h,q,v,{z+2 eK,k},{w0,w1}]]/;
  (Abs[Im[z]]>0 && Re[z]<0 && Precision[z]<Infinity)

(* Real variables "z" *)
GeneralizedLameWave[h_,q_,v_,{z_,k_},{w0_,w1_}]:=
  LameWave[h,q,v,{z,k},{w0,w1}]/;
  (N[Im[z]]===0. && Abs[Re[z]]<=N[EllipticK[k]] && Precision[z]<Infinity)

GeneralizedLameWave[h_,q_,v_,{z_,k_},{w0_,w1_}]:=
  Module[{eK,gK,gKm,gpK,gpKm,aKm,apKm,cons1,cons2},
    eK=N[EllipticK[k]];
    gK=initGWaveK[h,q,v,k,{w0,w1}];
    gKm=initGWaveKM[h,q,v,k,{w0,w1}];
    gpK=initGWavePrimeK[h,q,v,k,{w0,w1}];
    gpKm=initGWavePrimeKM[h,q,v,k,{w0,w1}];
    aKm=initAWaveKM[h,q,v,k,{w0,w1}];
    apKm=initAWavePrimeKM[h,q,v,k,{w0,w1}];
    cons1=(gK*apKm-gpK*aKm)/(gKm*apKm-gpKm*aKm);
    cons2=(gK*gpKm-gpK*gKm)/(aKm*gpKm-apKm*gKm);
    cons1*GeneralizedLameWave[h,q,v,{z-2 eK,k},{w0,w1}]+
    cons2*AuxiliaryLameWave[h,q,v,{z-2 eK,k},{w0,w1}]]/;
  (N[Im[z]]===0. && Re[z]>N[EllipticK[k]] && Precision[z]<Infinity)

GeneralizedLameWave[h_,q_,v_,{z_,k_},{w0_,w1_}]:=
  Module[{eK,gK,gKm,gpK,gpKm,aK,apK,cons1,cons2},
    eK=N[EllipticK[k]];
    gK=initGWaveK[h,q,v,k,{w0,w1}];
    gKm=initGWaveKM[h,q,v,k,{w0,w1}];
    gpK=initGWavePrimeK[h,q,v,k,{w0,w1}];
    gpKm=initGWavePrimeKM[h,q,v,k,{w0,w1}];
    aK=initAWaveK[h,q,v,k,{w0,w1}];
    apK=initAWavePrimeK[h,q,v,k,{w0,w1}];
    cons1=(gKm*apK-gpKm*aK)/(gK*apK-gpK*aK);
    cons2=(gKm*gpK-gpKm*gK)/(aK*gpK-apK*gK);
    cons1*GeneralizedLameWave[h,q,v,{z+2 eK,k},{w0,w1}]+
    cons2*AuxiliaryLameWave[h,q,v,{z+2 eK,k},{w0,w1}]]/;
  (N[Im[z]]===0. && Re[z]<-N[EllipticK[k]] && Precision[z]<Infinity)

(*--------- Auxiliary Lame wave function E2[h,v][z,k][w0,w1] ----------*)
(* Complex variables "z" *)
AuxiliaryLameWave[h_,q_,v_,{z_,k_},{w0_,w1_}]:=
  Module[{f0K,f1K},
    f0K=initLamWaveK[h,q,v,k,{w0-1,w1+1}];
    f1K=initLamWavePrimeK[h,q,v,k,{w0-1,w1+1}];
    LameWaveK[h,q,v,{z,k},{f0K,f1K}]]/;
  (Abs[Im[z]]>0 && 0<=Re[z]<=N[2*EllipticK[k]] && Precision[z]<Infinity)

AuxiliaryLameWave[h_,q_,v_,{z_,k_},{w0_,w1_}]:=
  Module[{eK,aK,aKm,apK,apKm,gKm,gpKm,cons1,cons2},
    eK=N[EllipticK[k]];
    aK=initAWaveK[h,q,v,k,{w0,w1}];
    aKm=initAWaveKM[h,q,v,k,{w0,w1}];
    apK=initAWavePrimeK[h,q,v,k,{w0,w1}];
    apKm=initAWavePrimeKM[h,q,v,k,{w0,w1}];
    gKm=initGWaveKM[h,q,v,k,{w0,w1}];
    gpKm=initGWavePrimeKM[h,q,v,k,{w0,w1}];
    cons1=(aK*gpKm-apK*gKm)/(aKm*gpKm-apKm*gKm);
    cons2=(aK*apKm-apK*aKm)/(gKm*apKm-gpKm*aKm);
    cons1*AuxiliaryLameWave[h,q,v,{z-2 eK,k},{w0,w1}]+
    cons2*GeneralizedLameWave[h,q,v,{z-2 eK,k},{w0,w1}]]/;
  (Abs[Im[z]]>0 && Re[z]>N[2*EllipticK[k]] && Precision[z]<Infinity)

AuxiliaryLameWave[h_,q_,v_,{z_,k_},{w0_,w1_}]:=
  Module[{eK,aK,aKm,apK,apKm,gK,gpK,cons1,cons2},
    eK=N[EllipticK[k]];
    aK=initAWaveK[h,q,v,k,{w0,w1}];
    aKm=initAWaveKM[h,q,v,k,{w0,w1}];
    apK=initAWavePrimeK[h,q,v,k,{w0,w1}];
    apKm=initAWavePrimeKM[h,q,v,k,{w0,w1}];
    gK=initGWaveK[h,q,v,k,{w0,w1}];
    gpK=initGWavePrimeK[h,q,v,k,{w0,w1}];
    cons1=(aKm*gpK-apKm*gK)/(aK*gpK-apK*gK);
    cons2=(aKm*apK-apKm*aK)/(gK*apK-gpK*aK);
    cons1*AuxiliaryLameWave[h,q,v,{z+2 eK,k},{w0,w1}]+
    cons2*GeneralizedLameWave[h,q,v,{z+2 eK,k},{w0,w1}]]/;
  (Abs[Im[z]]>0 && Re[z]<0 && Precision[z]<Infinity)

(* Real variables "z" *)
AuxiliaryLameWave[h_,q_,v_,{z_,k_},{w0_,w1_}]:=
  LameWave[h,q,v,{z,k},{w0-1,w1+1}]/;
  (N[Im[z]]===0. && Abs[Re[z]]<=N[EllipticK[k]] && Precision[z]<Infinity)

AuxiliaryLameWave[h_,q_,v_,{z_,k_},{w0_,w1_}]:=
  Module[{eK,aK,aKm,apK,apKm,gKm,gpKm,cons1,cons2},
    eK=N[EllipticK[k]];
    aK=initAWaveK[h,q,v,k,{w0,w1}];
    aKm=initAWaveKM[h,q,v,k,{w0,w1}];
    apK=initAWavePrimeK[h,q,v,k,{w0,w1}];
    apKm=initAWavePrimeKM[h,q,v,k,{w0,w1}];
    gKm=initGWaveKM[h,q,v,k,{w0,w1}];
    gpKm=initGWavePrimeKM[h,q,v,k,{w0,w1}];
    cons1=(aK*gpKm-apK*gKm)/(aKm*gpKm-apKm*gKm);
    cons2=(aK*apKm-apK*aKm)/(gKm*apKm-gpKm*aKm);
    cons1*AuxiliaryLameWave[h,q,v,{z-2 eK,k},{w0,w1}]+
    cons2*GeneralizedLameWave[h,q,v,{z-2 eK,k},{w0,w1}]]/;
  (N[Im[z]]===0. && Re[z]>N[EllipticK[k]] && Precision[z]<Infinity)

AuxiliaryLameWave[h_,q_,v_,{z_,k_},{w0_,w1_}]:=
  Module[{eK,aK,aKm,apK,apKm,gK,gpK,cons1,cons2},
    eK=N[EllipticK[k]];
    aK=initAWaveK[h,q,v,k,{w0,w1}];
    aKm=initAWaveKM[h,q,v,k,{w0,w1}];
    apK=initAWavePrimeK[h,q,v,k,{w0,w1}];
    apKm=initAWavePrimeKM[h,q,v,k,{w0,w1}];
    gK=initGWaveK[h,q,v,k,{w0,w1}];
    gpK=initGWavePrimeK[h,q,v,k,{w0,w1}];
    cons1=(aKm*gpK-apKm*gK)/(aK*gpK-apK*gK);
    cons2=(aKm*apK-apKm*aK)/(gK*apK-gpK*aK);
    cons1*AuxiliaryLameWave[h,q,v,{z+2 eK,k},{w0,w1}]+
    cons2*GeneralizedLameWave[h,q,v,{z+2 eK,k},{w0,w1}]]/;
  (N[Im[z]]===0. && Re[z]<-N[EllipticK[k]] && Precision[z]<Infinity)

(*--- Derivative Generalized Lame wave function E'[h,v][z,k][w0,w1] ---*)
(* Complex variables "z" *)
GeneralizedLameWavePrime[h_,q_,v_,{z_,k_},{w0_,w1_}]:=
  Module[{f0K,f1K},
    f0K=initLamWaveK[h,q,v,k,{w0,w1}];
    f1K=initLamWavePrimeK[h,q,v,k,{w0,w1}];
    LameWavePrimeK[h,q,v,{z,k},{f0K,f1K}]]/;
  (Abs[Im[z]]>0 && 0<=Re[z]<=N[2*EllipticK[k]] && Precision[z]<Infinity)

GeneralizedLameWavePrime[h_,q_,v_,{z_,k_},{w0_,w1_}]:=
  Module[{eK,gK,gKm,gpK,gpKm,aKm,apKm,cons1,cons2},
    eK=N[EllipticK[k]];
    gK=initGWaveK[h,q,v,k,{w0,w1}];
    gKm=initGWaveKM[h,q,v,k,{w0,w1}];
    gpK=initGWavePrimeK[h,q,v,k,{w0,w1}];
    gpKm=initGWavePrimeKM[h,q,v,k,{w0,w1}];
    aKm=initAWaveKM[h,q,v,k,{w0,w1}];
    apKm=initAWavePrimeKM[h,q,v,k,{w0,w1}];
    cons1=(gK*apKm-gpK*aKm)/(gKm*apKm-gpKm*aKm);
    cons2=(gK*gpKm-gpK*gKm)/(aKm*gpKm-apKm*gKm);
    cons1*GeneralizedLameWavePrime[h,q,v,{z-2 eK,k},{w0,w1}]+
    cons2*AuxiliaryLameWavePrime[h,q,v,{z-2 eK,k},{w0,w1}]]/;
  (Abs[Im[z]]>0 && Re[z]>N[2*EllipticK[k]] && Precision[z]<Infinity)

GeneralizedLameWavePrime[h_,q_,v_,{z_,k_},{w0_,w1_}]:=
  Module[{eK,gK,gKm,gpK,gpKm,aK,apK,cons1,cons2},
    eK=N[EllipticK[k]];
    gK=initGWaveK[h,q,v,k,{w0,w1}];
    gKm=initGWaveKM[h,q,v,k,{w0,w1}];
    gpK=initGWavePrimeK[h,q,v,k,{w0,w1}];
    gpKm=initGWavePrimeKM[h,q,v,k,{w0,w1}];
    aK=initAWaveK[h,q,v,k,{w0,w1}];
    apK=initAWavePrimeK[h,q,v,k,{w0,w1}];
    cons1=(gKm*apK-gpKm*aK)/(gK*apK-gpK*aK);
    cons2=(gKm*gpK-gpKm*gK)/(aK*gpK-apK*gK);
    cons1*GeneralizedLameWavePrime[h,q,v,{z+2 eK,k},{w0,w1}]+
    cons2*AuxiliaryLameWavePrime[h,q,v,{z+2 eK,k},{w0,w1}]]/;
  (Abs[Im[z]]>0 && Re[z]<0 && Precision[z]<Infinity)

(* Real variables "z" *)
GeneralizedLameWavePrime[h_,q_,v_,{z_,k_},{w0_,w1_}]:=
  LameWavePrime[h,q,v,{z,k},{w0,w1}]/;
  (N[Im[z]]===0. && Abs[Re[z]]<=N[EllipticK[k]] && Precision[z]<Infinity)

GeneralizedLameWavePrime[h_,q_,v_,{z_,k_},{w0_,w1_}]:=
  Module[{eK,gK,gKm,gpK,gpKm,aKm,apKm,cons1,cons2},
    eK=N[EllipticK[k]];
    gK=initGWaveK[h,q,v,k,{w0,w1}];
    gKm=initGWaveKM[h,q,v,k,{w0,w1}];
    gpK=initGWavePrimeK[h,q,v,k,{w0,w1}];
    gpKm=initGWavePrimeKM[h,q,v,k,{w0,w1}];
    aKm=initAWaveKM[h,q,v,k,{w0,w1}];
    apKm=initAWavePrimeKM[h,q,v,k,{w0,w1}];
    cons1=(gK*apKm-gpK*aKm)/(gKm*apKm-gpKm*aKm);
    cons2=(gK*gpKm-gpK*gKm)/(aKm*gpKm-apKm*gKm);
    cons1*GeneralizedLameWavePrime[h,q,v,{z-2 eK,k},{w0,w1}]+
    cons2*AuxiliaryLameWavePrime[h,q,v,{z-2 eK,k},{w0,w1}]]/;
  (N[Im[z]]===0. && Re[z]>N[EllipticK[k]] && Precision[z]<Infinity)

GeneralizedLameWavePrime[h_,q_,v_,{z_,k_},{w0_,w1_}]:=
  Module[{eK,gK,gKm,gpK,gpKm,aK,apK,cons1,cons2},
    eK=N[EllipticK[k]];
    gK=initGWaveK[h,q,v,k,{w0,w1}];
    gKm=initGWaveKM[h,q,v,k,{w0,w1}];
    gpK=initGWavePrimeK[h,q,v,k,{w0,w1}];
    gpKm=initGWavePrimeKM[h,q,v,k,{w0,w1}];
    aK=initAWaveK[h,q,v,k,{w0,w1}];
    apK=initAWavePrimeK[h,q,v,k,{w0,w1}];
    cons1=(gKm*apK-gpKm*aK)/(gK*apK-gpK*aK);
    cons2=(gKm*gpK-gpKm*gK)/(aK*gpK-apK*gK);
    cons1*GeneralizedLameWavePrime[h,q,v,{z+2 eK,k},{w0,w1}]+
    cons2*AuxiliaryLameWavePrime[h,q,v,{z+2 eK,k},{w0,w1}]]/;
  (N[Im[z]]===0. && Re[z]<-N[EllipticK[k]] && Precision[z]<Infinity)

(*--- Derivative Auxiliary Lame wave function E2'[h,v][z,k][w0,w1] ----*)
(* Complex variables "z" *)
AuxiliaryLameWavePrime[h_,q_,v_,{z_,k_},{w0_,w1_}]:=
  Module[{f0K,f1K},
    f0K=initLamWaveK[h,q,v,k,{w0-1,w1+1}];
    f1K=initLamWavePrimeK[h,q,v,k,{w0-1,w1+1}];
    LameWavePrimeK[h,q,v,{z,k},{f0K,f1K}]]/;
  (Abs[Im[z]]>0 && 0<=Re[z]<=N[2*EllipticK[k]] && Precision[z]<Infinity)

AuxiliaryLameWavePrime[h_,q_,v_,{z_,k_},{w0_,w1_}]:=
  Module[{eK,aK,aKm,apK,apKm,gKm,gpKm,cons1,cons2},
    eK=N[EllipticK[k]];
    aK=initAWaveK[h,q,v,k,{w0,w1}];
    aKm=initAWaveKM[h,q,v,k,{w0,w1}];
    apK=initAWavePrimeK[h,q,v,k,{w0,w1}];
    apKm=initAWavePrimeKM[h,q,v,k,{w0,w1}];
    gKm=initGWaveKM[h,q,v,k,{w0,w1}];
    gpKm=initGWavePrimeKM[h,q,v,k,{w0,w1}];
    cons1=(aK*gpKm-apK*gKm)/(aKm*gpKm-apKm*gKm);
    cons2=(aK*apKm-apK*aKm)/(gKm*apKm-gpKm*aKm);
    cons1*AuxiliaryLameWavePrime[h,q,v,{z-2 eK,k},{w0,w1}]+
    cons2*GeneralizedLameWavePrime[h,q,v,{z-2 eK,k},{w0,w1}]]/;
  (Abs[Im[z]]>0 && Re[z]>N[2*EllipticK[k]] && Precision[z]<Infinity)

AuxiliaryLameWavePrime[h_,q_,v_,{z_,k_},{w0_,w1_}]:=
  Module[{eK,aK,aKm,apK,apKm,gK,gpK,cons1,cons2},
    eK=N[EllipticK[k]];
    aK=initAWaveK[h,q,v,k,{w0,w1}];
    aKm=initAWaveKM[h,q,v,k,{w0,w1}];
    apK=initAWavePrimeK[h,q,v,k,{w0,w1}];
    apKm=initAWavePrimeKM[h,q,v,k,{w0,w1}];
    gK=initGWaveK[h,q,v,k,{w0,w1}];
    gpK=initGWavePrimeK[h,q,v,k,{w0,w1}];
    cons1=(aKm*gpK-apKm*gK)/(aK*gpK-apK*gK);
    cons2=(aKm*apK-apKm*aK)/(gK*apK-gpK*aK);
    cons1*AuxiliaryLameWavePrime[h,q,v,{z+2 eK,k},{w0,w1}]+
    cons2*GeneralizedLameWavePrime[h,q,v,{z+2 eK,k},{w0,w1}]]/;
  (Abs[Im[z]]>0 && Re[z]<0 && Precision[z]<Infinity)

(* Real variables "z" *)
AuxiliaryLameWavePrime[h_,q_,v_,{z_,k_},{w0_,w1_}]:=
  LameWavePrime[h,q,v,{z,k},{w0-1,w1+1}]/;
  (N[Im[z]]===0. && Abs[Re[z]]<=N[EllipticK[k]] && Precision[z]<Infinity)

AuxiliaryLameWavePrime[h_,q_,v_,{z_,k_},{w0_,w1_}]:=
  Module[{eK,aK,aKm,apK,apKm,gKm,gpKm,cons1,cons2},
    eK=N[EllipticK[k]];
    aK=initAWaveK[h,q,v,k,{w0,w1}];
    aKm=initAWaveKM[h,q,v,k,{w0,w1}];
    apK=initAWavePrimeK[h,q,v,k,{w0,w1}];
    apKm=initAWavePrimeKM[h,q,v,k,{w0,w1}];
    gKm=initGWaveKM[h,q,v,k,{w0,w1}];
    gpKm=initGWavePrimeKM[h,q,v,k,{w0,w1}];
    cons1=(aK*gpKm-apK*gKm)/(aKm*gpKm-apKm*gKm);
    cons2=(aK*apKm-apK*aKm)/(gKm*apKm-gpKm*aKm);
    cons1*AuxiliaryLameWavePrime[h,q,v,{z-2 eK,k},{w0,w1}]+
    cons2*GeneralizedLameWavePrime[h,q,v,{z-2 eK,k},{w0,w1}]]/;
  (N[Im[z]]===0. && Re[z]>N[EllipticK[k]] && Precision[z]<Infinity)

AuxiliaryLameWavePrime[h_,q_,v_,{z_,k_},{w0_,w1_}]:=
  Module[{eK,aK,aKm,apK,apKm,gK,gpK,cons1,cons2},
    eK=N[EllipticK[k]];
    aK=initAWaveK[h,q,v,k,{w0,w1}];
    aKm=initAWaveKM[h,q,v,k,{w0,w1}];
    apK=initAWavePrimeK[h,q,v,k,{w0,w1}];
    apKm=initAWavePrimeKM[h,q,v,k,{w0,w1}];
    gK=initGWaveK[h,q,v,k,{w0,w1}];
    gpK=initGWavePrimeK[h,q,v,k,{w0,w1}];
    cons1=(aKm*gpK-apKm*gK)/(aK*gpK-apK*gK);
    cons2=(aKm*apK-apKm*aK)/(gK*apK-gpK*aK);
    cons1*AuxiliaryLameWavePrime[h,q,v,{z+2 eK,k},{w0,w1}]+
    cons2*GeneralizedLameWavePrime[h,q,v,{z+2 eK,k},{w0,w1}]]/;
  (N[Im[z]]===0. && Re[z]<-N[EllipticK[k]] && Precision[z]<Infinity)



(**********************************************************************************)

End[]

Protect[Hill, HillPrime, HillEllipticTheta, HillEllipticThetaPrime, 
        HillSinCos, HillSinCosPrime, HillCosSin, HillCosSinPrime, 
        HillCosCos, HillCosCosPrime, MeissnerHill, MeissnerHillPrime, 
        LameEigenvalueHC, LameEigenvalueHS, LameEigenvalueHC2, LameEigenvalueHS2, 
        LameEigenvalueA, LameEigenvalueB, LameEigenvalueA2, LameEigenvalueB2, 
        LameEC, LameES, LameECPrime, LameESPrime, 
        LameFC, LameFS, LameFCPrime, LameFSPrime, 
        LameLC, LameLS, LameLCPrime, LameLSPrime, 
        LamePolynomialUE, LamePolynomialSE, LamePolynomialCE, 
        LamePolynomialDE, LamePolynomialSCE, LamePolynomialSDE, 
        LamePolynomialCDE, LamePolynomialSCDE, 
        LamePolynomialUEPrime, LamePolynomialSEPrime, LamePolynomialCEPrime, 
        LamePolynomialDEPrime, LamePolynomialSCEPrime, LamePolynomialSDEPrime, 
        LamePolynomialCDEPrime, LamePolynomialSCDEPrime, 
        GeneralizedLame, GeneralizedLamePrime, 
        GeneralizedLameWave, GeneralizedLameWavePrime]

EndPackage[]
